@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: conf.h 5302 2008-03-19 14:54:32Z charles $
+ * $Id: conf.h 7722 2009-01-16 17:31:10Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -22,26 +22,26 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
-/**
-***
-**/
-
 #ifndef TG_CONF_H
 #define TG_CONF_H
 
-int       pref_int_get            ( const char * key );
-void      pref_int_set            ( const char * key, int value );
-void      pref_int_set_default    ( const char * key, int value );
+#include <inttypes.h>
+#include <libtransmission/transmission.h>
 
-gboolean  pref_flag_get           ( const char * key );
-void      pref_flag_set           ( const char * key, gboolean value );
-void      pref_flag_set_default   ( const char * key, gboolean value );
+int64_t          pref_int_get           ( const char * key );
+void             pref_int_set           ( const char * key, int64_t value );
+void             pref_int_set_default   ( const char * key, int64_t value );
 
-char*     pref_string_get         ( const char * key );
-void      pref_string_set         ( const char * key, const char * value );
-void      pref_string_set_default ( const char * key, const char * value );
+gboolean         pref_flag_get          ( const char * key );
+void             pref_flag_set          ( const char * key, gboolean value );
+void             pref_flag_set_default  ( const char * key, gboolean value );
 
-void      pref_save               ( char **errstr );
+const char*      pref_string_get        ( const char * key );
+void             pref_string_set        ( const char * key, const char * value );
+void             pref_string_set_default( const char * key, const char * value );
+
+void             pref_save              ( tr_session * );
+struct tr_benc*  pref_get_all           ( void );
 
 /**
 ***
@@ -56,20 +56,20 @@ enum
 
 typedef int pref_flag_t;
 
-gboolean pref_flag_eval( pref_flag_t val, const char * key );
+gboolean pref_flag_eval( pref_flag_t  val,
+                         const char * key );
 
 
 /**
 ***
 **/
 
-gboolean
-cf_init(const char *confdir, char **errstr);
-gboolean
-cf_lock(char **errstr);
-char *
-cf_sockname(void);
-void
-cf_check_older_configs(void);
+gboolean cf_init( const char *confdir,
+                  char **     errstr );
+
+gboolean cf_lock( tr_lockfile_state_t *tr_state,
+                  char **              errstr );
+
+void     cf_check_older_configs( void );
 
 #endif /* TG_CONF_H */
