@@ -1,21 +1,22 @@
 /*
- * This file Copyright (C) 2007-2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license. 
+ * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
- * 
- * $Id: details.h 5122 2008-02-26 19:58:03Z charles $
+ *
+ * $Id: details.h 7664 2009-01-11 17:46:51Z charles $
  */
 
 #ifndef GTK_TORRENT_INSPECTOR_H
 #define GTK_TORRENT_INSPECTOR_H
 
-#include <gtk/gtkwindow.h>
+#include <gtk/gtk.h>
 #include "tr-torrent.h"
 
-GtkWidget* torrent_inspector_new ( GtkWindow * parent, TrTorrent * tor );
+GtkWidget* torrent_inspector_new( GtkWindow * parent,
+                                  TrTorrent * tor );
 
 #endif /* TG_PREFS_H */
