@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2007-2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license. 
+ * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
- * 
- * $Id: makemeta-ui.h 4404 2008-01-01 17:20:20Z charles $
+ *
+ * $Id: makemeta-ui.h 7664 2009-01-11 17:46:51Z charles $
  */
 
 #ifndef MAKE_META_UI__H
@@ -16,6 +16,7 @@
 #include <gtk/gtk.h>
 #include <libtransmission/transmission.h>
 
-GtkWidget* make_meta_ui( GtkWindow * parent, tr_handle * handle );
+GtkWidget* make_meta_ui( GtkWindow  * parent,
+                         tr_session * session );
 
 #endif
