@@ -20,7 +20,9 @@
  */
 #include <string.h>
 #include <gtk/gtk.h>
+
 #include "sexy-icon-entry.h"
+#include "sexy-marshal.h"
 
 #define ICON_MARGIN 2
 #define MAX_ICONS 2
@@ -89,12 +91,14 @@ sexy_icon_entry_class_init(SexyIconEntryClass *klass)
 	GObjectClass *gobject_class;
 	GtkObjectClass *object_class;
 	GtkWidgetClass *widget_class;
+	GtkEntryClass *entry_class;
 
 	parent_class = g_type_class_peek_parent(klass);
 
 	gobject_class = G_OBJECT_CLASS(klass);
 	object_class  = GTK_OBJECT_CLASS(klass);
 	widget_class  = GTK_WIDGET_CLASS(klass);
+	entry_class   = GTK_ENTRY_CLASS(klass);
 
 	gobject_class->finalize = sexy_icon_entry_finalize;
 
@@ -126,7 +130,7 @@ sexy_icon_entry_class_init(SexyIconEntryClass *klass)
 					 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 					 G_STRUCT_OFFSET(SexyIconEntryClass, icon_pressed),
 					 NULL, NULL,
-					 gtk_marshal_VOID__INT_INT,
+					 sexy_marshal_VOID__INT_INT,
 					 G_TYPE_NONE, 2,
 					 G_TYPE_INT,
 					 G_TYPE_INT);
@@ -146,14 +150,14 @@ sexy_icon_entry_class_init(SexyIconEntryClass *klass)
 					 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 					 G_STRUCT_OFFSET(SexyIconEntryClass, icon_released),
 					 NULL, NULL,
-					 gtk_marshal_VOID__INT_INT,
+					 sexy_marshal_VOID__INT_INT,
 					 G_TYPE_NONE, 2,
 					 G_TYPE_INT,
 					 G_TYPE_INT);
 }
 
 static void
-sexy_icon_entry_editable_init(GtkEditableClass *iface G_GNUC_UNUSED )
+sexy_icon_entry_editable_init(GtkEditableClass *iface G_GNUC_UNUSED)
 {
 };
 
@@ -387,10 +391,12 @@ sexy_icon_entry_unrealize(GtkWidget *widget)
 static void
 sexy_icon_entry_size_request(GtkWidget *widget, GtkRequisition *requisition)
 {
+	GtkEntry *gtkentry;
 	SexyIconEntry *entry;
 	gint icon_widths = 0;
 	int i;
 
+	gtkentry = GTK_ENTRY(widget);
 	entry    = SEXY_ICON_ENTRY(widget);
 
 	for (i = 0; i < MAX_ICONS; i++)
@@ -464,9 +470,10 @@ static GdkPixbuf *
 get_pixbuf_from_icon(SexyIconEntry *entry, SexyIconEntryPosition icon_pos)
 {
 	GdkPixbuf *pixbuf = NULL;
-	gchar *stock_id;
+	const gchar *stock_id;
 	SexyIconInfo *icon_info = &entry->priv->icons[icon_pos];
 	GtkIconSize size;
+	int w, h;
 
 	switch (gtk_image_get_storage_type(GTK_IMAGE(icon_info->icon)))
 	{
@@ -476,11 +483,17 @@ get_pixbuf_from_icon(SexyIconEntry *entry, SexyIconEntryPosition icon_pos)
 			break;
 
 		case GTK_IMAGE_STOCK:
-			gtk_image_get_stock(GTK_IMAGE(icon_info->icon), &stock_id, &size);
+			gtk_image_get_stock(GTK_IMAGE(icon_info->icon), (char**)&stock_id, &size);
 			pixbuf = gtk_widget_render_icon(GTK_WIDGET(entry),
 											stock_id, size, NULL);
 			break;
 
+		case GTK_IMAGE_ICON_NAME:
+			gtk_image_get_icon_name (GTK_IMAGE(icon_info->icon), &stock_id, &size);
+			gtk_icon_size_lookup (size, &w, &h);
+			pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), stock_id, size, 0, NULL);
+			break;
+          
 		default:
 			return NULL;
 	}
@@ -566,7 +579,8 @@ draw_icon(GtkWidget *widget, SexyIconEntryPosition icon_pos)
 
 		scale = height - (2 * ICON_MARGIN);
 
-		temp_pixbuf = gdk_pixbuf_scale_simple(pixbuf, scale, scale, GDK_INTERP_BILINEAR);
+		temp_pixbuf = gdk_pixbuf_scale_simple(pixbuf, scale, scale,
+											  GDK_INTERP_BILINEAR);
 
 		g_object_unref(pixbuf);
 
@@ -650,7 +664,8 @@ update_icon(GObject *obj G_GNUC_UNUSED, GParamSpec *param, SexyIconEntry *entry)
 
 		if (strcmp(name, "pixbuf")   && strcmp(name, "stock")  &&
 			strcmp(name, "image")    && strcmp(name, "pixmap") &&
-			strcmp(name, "icon_set") && strcmp(name, "pixbuf_animation"))
+			strcmp(name, "icon-set") && strcmp(name, "pixbuf-animation") &&
+			strcmp(name, "icon-name"))
 		{
 			return;
 		}
