@@ -1,16 +1,18 @@
-/*
- * This file Copyright (C) 2007-2008 Charles Kerr <charles@rebelbase.com>
+/* * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license. 
+ * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: crypto.c 6479 2008-08-11 04:40:28Z charles $
+ * $Id: crypto.c 7873 2009-02-11 16:07:45Z charles $
  */
 
-#include <sys/types.h> /* for event.h, as well as netinet/in.h on some platforms */
+#include <stdlib.h> /* for abs() */
+#include <limits.h> /* for INT_MAX */
+#include <sys/types.h> /* for event.h, as well as netinet/in.h on some platforms
+                         */
 #include <assert.h>
 #include <inttypes.h> /* uint8_t */
 #include <string.h> /* memcpy */
@@ -18,20 +20,24 @@
 
 #include <openssl/bn.h>
 #include <openssl/dh.h>
+#include <openssl/err.h>
 #include <openssl/rc4.h>
 #include <openssl/sha.h>
+#include <openssl/rand.h>
 
 #include <event.h>
 
 #include "crypto.h"
 #include "utils.h"
 
+#define MY_NAME "tr_crypto"
+
 /**
 ***
 **/
 
 void
-tr_sha1( uint8_t    * setme,
+tr_sha1( uint8_t *    setme,
          const void * content1,
          int          content1_len,
          ... )
@@ -43,10 +49,11 @@ tr_sha1( uint8_t    * setme,
     SHA1_Update( &sha, content1, content1_len );
 
     va_start( vl, content1_len );
-    for( ;; ) {
+    for( ; ; )
+    {
         const void * content = (const void*) va_arg( vl, const void* );
-        const int content_len = content ? (int) va_arg( vl, int ) : -1;
-        if( content==NULL || content_len<1 )
+        const int    content_len = content ? (int) va_arg( vl, int ) : -1;
+        if( content == NULL || content_len < 1 )
             break;
         SHA1_Update( &sha, content, content_len );
     }
@@ -78,20 +85,29 @@ static const uint8_t dh_G[] = { 2 };
 
 struct tr_crypto
 {
-    RC4_KEY dec_key;
-    RC4_KEY enc_key;
-    uint8_t torrentHash[SHA_DIGEST_LENGTH];
-    unsigned int isIncoming       : 1;
-    unsigned int torrentHashIsSet : 1;
-    unsigned int mySecretIsSet    : 1;
-    uint8_t myPublicKey[KEY_LEN];
-    uint8_t mySecret[KEY_LEN];
+    RC4_KEY         dec_key;
+    RC4_KEY         enc_key;
+    uint8_t         torrentHash[SHA_DIGEST_LENGTH];
+    tr_bool         isIncoming;
+    tr_bool         torrentHashIsSet;
+    tr_bool         mySecretIsSet;
+    uint8_t         myPublicKey[KEY_LEN];
+    uint8_t         mySecret[KEY_LEN];
 };
 
 /**
 ***
 **/
 
+#define logErrorFromSSL( ... ) \
+    do { \
+        if( tr_msgLoggingIsActive( TR_MSG_ERR ) ) { \
+            char buf[512]; \
+            ERR_error_string_n( ERR_get_error( ), buf, sizeof( buf ) ); \
+            tr_msg( __FILE__, __LINE__, TR_MSG_ERR, MY_NAME, "%s", buf ); \
+        } \
+    } while( 0 )
+
 static DH*
 getSharedDH( void )
 {
@@ -100,21 +116,29 @@ getSharedDH( void )
     if( dh == NULL )
     {
         dh = DH_new( );
-        dh->p = BN_bin2bn( dh_P, sizeof(dh_P), NULL );
-        dh->g = BN_bin2bn( dh_G, sizeof(dh_G), NULL );
-        DH_generate_key( dh );
+
+        dh->p = BN_bin2bn( dh_P, sizeof( dh_P ), NULL );
+        if( dh->p == NULL )
+            logErrorFromSSL( );
+
+        dh->g = BN_bin2bn( dh_G, sizeof( dh_G ), NULL );
+        if( dh->g == NULL )
+            logErrorFromSSL( );
+
+        if( !DH_generate_key( dh ) )
+            logErrorFromSSL( );
     }
 
     return dh;
 }
 
-tr_crypto * 
+tr_crypto *
 tr_cryptoNew( const uint8_t * torrentHash,
               int             isIncoming )
 {
-    int len, offset;
+    int         len, offset;
     tr_crypto * crypto;
-    DH * dh = getSharedDH( );
+    DH *        dh = getSharedDH( );
 
     crypto = tr_new0( tr_crypto, 1 );
     crypto->isIncoming = isIncoming ? 1 : 0;
@@ -143,29 +167,35 @@ tr_cryptoFree( tr_crypto * crypto )
 **/
 
 const uint8_t*
-tr_cryptoComputeSecret( tr_crypto      * crypto,
-                        const uint8_t  * peerPublicKey )
+tr_cryptoComputeSecret( tr_crypto *     crypto,
+                        const uint8_t * peerPublicKey )
 {
-    int len, offset;
-    uint8_t secret[KEY_LEN];
+    int      len;
+    uint8_t  secret[KEY_LEN];
     BIGNUM * bn = BN_bin2bn( peerPublicKey, KEY_LEN, NULL );
-    DH * dh = getSharedDH( );
-    assert( DH_size(dh) == KEY_LEN );
+    DH *     dh = getSharedDH( );
+
+    assert( DH_size( dh ) == KEY_LEN );
 
     len = DH_compute_key( secret, bn, dh );
-    assert( len <= KEY_LEN );
-    offset = KEY_LEN - len;
-    memset( crypto->mySecret, 0, offset );
-    memcpy( crypto->mySecret + offset, secret, len );
-    crypto->mySecretIsSet = 1;
-    
-    BN_free( bn );
+    if( len == -1 )
+        logErrorFromSSL( );
+    else {
+        int offset;
+        assert( len <= KEY_LEN );
+        offset = KEY_LEN - len;
+        memset( crypto->mySecret, 0, offset );
+        memcpy( crypto->mySecret + offset, secret, len );
+        crypto->mySecretIsSet = 1;
+    }
 
+    BN_free( bn );
     return crypto->mySecret;
 }
 
 const uint8_t*
-tr_cryptoGetMyPublicKey( const tr_crypto * crypto, int * setme_len )
+tr_cryptoGetMyPublicKey( const tr_crypto * crypto,
+                         int *             setme_len )
 {
     *setme_len = KEY_LEN;
     return crypto->myPublicKey;
@@ -176,7 +206,9 @@ tr_cryptoGetMyPublicKey( const tr_crypto * crypto, int * setme_len )
 **/
 
 static void
-initRC4( tr_crypto * crypto, RC4_KEY * setme, const char * key )
+initRC4( tr_crypto *  crypto,
+         RC4_KEY *    setme,
+         const char * key )
 {
     SHA_CTX sha;
     uint8_t buf[SHA_DIGEST_LENGTH];
@@ -184,28 +216,35 @@ initRC4( tr_crypto * crypto, RC4_KEY * setme, const char * key )
     assert( crypto->torrentHashIsSet );
     assert( crypto->mySecretIsSet );
 
-    SHA1_Init( &sha );
-    SHA1_Update( &sha, key, 4 );
-    SHA1_Update( &sha, crypto->mySecret, KEY_LEN );
-    SHA1_Update( &sha, crypto->torrentHash, SHA_DIGEST_LENGTH );
-    SHA1_Final( buf, &sha );
-    RC4_set_key( setme, SHA_DIGEST_LENGTH, buf );
+    if( SHA1_Init( &sha )
+        && SHA1_Update( &sha, key, 4 )
+        && SHA1_Update( &sha, crypto->mySecret, KEY_LEN )
+        && SHA1_Update( &sha, crypto->torrentHash, SHA_DIGEST_LENGTH )
+        && SHA1_Final( buf, &sha ) )
+    {
+        RC4_set_key( setme, SHA_DIGEST_LENGTH, buf );
+    }
+    else
+    {
+        logErrorFromSSL( );
+    }
 }
 
 void
 tr_cryptoDecryptInit( tr_crypto * crypto )
 {
     unsigned char discard[1024];
-    const char * txt = crypto->isIncoming ? "keyA" : "keyB";
+    const char *  txt = crypto->isIncoming ? "keyA" : "keyB";
+
     initRC4( crypto, &crypto->dec_key, txt );
-    RC4( &crypto->dec_key, sizeof(discard), discard, discard );
+    RC4( &crypto->dec_key, sizeof( discard ), discard, discard );
 }
 
 void
-tr_cryptoDecrypt( tr_crypto  * crypto,
+tr_cryptoDecrypt( tr_crypto *  crypto,
                   size_t       buf_len,
                   const void * buf_in,
-                  void       * buf_out )
+                  void *       buf_out )
 {
     RC4( &crypto->dec_key, buf_len,
          (const unsigned char*)buf_in,
@@ -216,16 +255,17 @@ void
 tr_cryptoEncryptInit( tr_crypto * crypto )
 {
     unsigned char discard[1024];
-    const char * txt = crypto->isIncoming ? "keyB" : "keyA";
+    const char *  txt = crypto->isIncoming ? "keyB" : "keyA";
+
     initRC4( crypto, &crypto->enc_key, txt );
-    RC4( &crypto->enc_key, sizeof(discard), discard, discard );
+    RC4( &crypto->enc_key, sizeof( discard ), discard, discard );
 }
 
 void
-tr_cryptoEncrypt( tr_crypto  * crypto,
+tr_cryptoEncrypt( tr_crypto *  crypto,
                   size_t       buf_len,
                   const void * buf_in,
-                  void       * buf_out )
+                  void *       buf_out )
 {
     RC4( &crypto->enc_key, buf_len,
          (const unsigned char*)buf_in,
@@ -237,7 +277,7 @@ tr_cryptoEncrypt( tr_crypto  * crypto,
 **/
 
 void
-tr_cryptoSetTorrentHash( tr_crypto     * crypto,
+tr_cryptoSetTorrentHash( tr_crypto *     crypto,
                          const uint8_t * hash )
 {
     crypto->torrentHashIsSet = hash ? 1 : 0;
@@ -264,3 +304,48 @@ tr_cryptoHasTorrentHash( const tr_crypto * crypto )
 
     return crypto->torrentHashIsSet ? 1 : 0;
 }
+
+int
+tr_cryptoRandInt( int upperBound )
+{
+    int noise;
+    int val;
+
+    if( RAND_pseudo_bytes ( (unsigned char *) &noise, sizeof noise ) >= 0 )
+    {
+        val = abs( noise ) % upperBound;
+    }
+    else /* fall back to a weaker implementation... */
+    {
+        val = tr_cryptoWeakRandInt( upperBound );
+    }
+
+    assert( val >= 0 );
+    assert( val < upperBound );
+    return val;
+}
+
+int
+tr_cryptoWeakRandInt( int upperBound )
+{
+    static int init = 0;
+
+    assert( upperBound > 0 );
+
+    if( !init )
+    {
+        srand( tr_date( ) );
+        init = 1;
+    }
+
+    return rand( ) % upperBound;
+}
+
+void
+tr_cryptoRandBuf( unsigned char *buf,
+                  size_t         len )
+{
+    if( RAND_pseudo_bytes ( buf, len ) != 1 )
+        logErrorFromSSL( );
+}
+
