@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: fdlimit.h 4734 2008-01-18 19:13:32Z charles $
+ * $Id: fdlimit.h 7722 2009-01-16 17:31:10Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -22,16 +22,15 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
+#ifndef __TRANSMISSION__
+#error only libtransmission should #include this header.
+#endif
+
+#include "transmission.h"
 #include "net.h"
 
-/***********************************************************************
- * tr_fdInit
- ***********************************************************************
- * Detect the maximum number of open files and initializes things.
- **********************************************************************/
-void tr_fdInit( int globalPeerLimit );
-
-void tr_fdClose( void );
+void tr_fdInit( size_t openFileLimit,
+                size_t globalPeerLimit );
 
 /**
  * Returns an fd to the specified filename.
@@ -42,20 +41,21 @@ void tr_fdClose( void );
  * write to the file at a time.  Callers check out a file, use it,
  * and then check it back in via tr_fdFileReturn() when done.
  *
- * - if `folder' doesn't exist, TR_ERROR_IO_PARENT is returned.
+ * - if `folder' doesn't exist, errno is set to ENOENT.
  * - if doWrite is true, subfolders in torrentFile are created if necessary.
  * - if doWrite is true, the target file is created if necessary.
  *
  * on success, a file descriptor >= 0 is returned.
- * on failure, a negative number corresponding to tr_errno is returned.
+ * on failure, a -1 is returned and errno is set.
  *
  * @see tr_fdFileReturn
  * @see tr_fdFileClose
- * @see tr_errno
  */
-int tr_fdFileCheckout( const char * folder,
-                       const char * torrentFile,
-                       int          doWrite );
+int  tr_fdFileCheckout( const char             * folder,
+                        const char             * torrentFile,
+                        tr_bool                  doWrite,
+                        tr_preallocation_mode    preallocationMode,
+                        uint64_t                 desiredFileSize );
 
 /**
  * Returns an fd from tr_fdFileCheckout() so that other clients may borrow it.
@@ -74,26 +74,29 @@ void tr_fdFileReturn( int file );
  * @see tr_fdFileCheckout
  * @see tr_fdFileReturn
  */
-void tr_fdFileClose( const char * filename );
-
+void     tr_fdFileClose( const char * filename );
 
 
 /***********************************************************************
  * Sockets
  **********************************************************************/
-int  tr_fdSocketCreate( int type, int priority );
-int  tr_fdSocketAccept( int b, struct in_addr * addr, tr_port_t * port );
-void tr_fdSocketClose( int s );
+int      tr_fdSocketCreate( int domain, int type );
+
+int      tr_fdSocketAccept( int           b,
+                            tr_address  * addr,
+                            tr_port     * port );
+
+void     tr_fdSocketClose( int s );
 
 /***********************************************************************
  * tr_fdClose
  ***********************************************************************
  * Frees resources allocated by tr_fdInit.
  **********************************************************************/
-void tr_fdClose( void );
+void     tr_fdClose( void );
 
 
-void tr_fdSetPeerLimit( uint16_t n );
+void     tr_fdSetPeerLimit( uint16_t n );
 
 uint16_t tr_fdGetPeerLimit( void );
 
