@@ -1,3 +1,8 @@
+#ifndef __TRANSMISSION__
+#error only libtransmission should #include this header.
+#endif
+
+
 /* File ggets.h  - goodgets is a safe alternative to gets */
 /* By C.B. Falconer.  Public domain 2002-06-22            */
 /*    attribution appreciated.                            */
@@ -28,18 +33,19 @@
  */
 
 #ifndef ggets_h_
-#  define ggets_h_
+#define ggets_h_
 
-#  ifdef __cplusplus
-      extern "C" {
-#  endif
+#ifdef __cplusplus
+extern "C" {
+#endif
 
-int fggets(char* *ln, FILE *f);
+int fggets( char* *ln,
+            FILE * f );
 
-#define ggets(ln) fggets(ln, stdin)
+#define ggets( ln ) fggets( ln, stdin )
 
-#  ifdef __cplusplus
-      }
-#  endif
+#ifdef __cplusplus
+}
+#endif
 #endif
 /* END ggets.h */
