@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ratecontrol.c 6425 2008-08-01 16:43:22Z charles $
+ * $Id: ratecontrol.c 7664 2009-01-11 17:46:51Z charles $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -29,91 +29,41 @@
 #include "ratecontrol.h"
 #include "utils.h"
 
-#define GRANULARITY_MSEC 500
-#define SHORT_INTERVAL_MSEC 1000
-#define LONG_INTERVAL_MSEC 8000
-#define HISTORY_SIZE (LONG_INTERVAL_MSEC / GRANULARITY_MSEC)
-
-struct tr_transfer
-{
-    uint64_t date;
-    uint64_t size;
-};
-
-struct tr_ratecontrol
-{
-    int limit;
-    int newest;
-    struct tr_transfer transfers[HISTORY_SIZE];
-};
-
 /* return the xfer rate over the last `interval' seconds in KiB/sec */
 static float
-rateForInterval( const tr_ratecontrol * r, int interval_msec )
+rateForInterval( const tr_ratecontrol * r,
+                 int                    interval_msec,
+                 uint64_t               now )
 {
-    uint64_t bytes = 0;
-    const uint64_t cutoff = tr_date () - interval_msec;
-    int i = r->newest;
-    for( ;; )
+    uint64_t       bytes = 0;
+    const uint64_t cutoff = (now?now:tr_date()) - interval_msec;
+    int            i = r->newest;
+
+    for( ; ; )
     {
         if( r->transfers[i].date <= cutoff )
             break;
 
         bytes += r->transfers[i].size;
 
-        if( --i == -1 ) i = HISTORY_SIZE - 1; /* circular history */
+        if( --i == -1 ) i = TR_RC_HISTORY_SIZE - 1; /* circular history */
         if( i == r->newest ) break; /* we've come all the way around */
     }
 
-    return (bytes/1024.0) * (1000.0/interval_msec);
+    return ( bytes / 1024.0 ) * ( 1000.0 / interval_msec );
 }
 
 /***
 ****
 ***/
 
-tr_ratecontrol*
-tr_rcInit( void )
-{
-    tr_ratecontrol * r = tr_new0( tr_ratecontrol, 1 );
-    r->limit = 0;
-    return r;
-}
-
-void
-tr_rcClose( tr_ratecontrol * r )
-{
-    tr_rcReset( r );
-    tr_free( r );
-}
-
-/***
-****
-***/
-
-size_t
-tr_rcBytesLeft( const tr_ratecontrol * r )
-{
-    size_t bytes = 0;
-
-    if( r )
-    {
-        const float cur = rateForInterval( r, SHORT_INTERVAL_MSEC );
-        const float max = r->limit;
-        const float kb = max>cur ? max-cur : 0;
-        bytes = (size_t)(kb * 1024);
-    }
-
-    return bytes;
-}
-
 float
-tr_rcRate( const tr_ratecontrol * r )
+tr_rcRate( const tr_ratecontrol * r, uint64_t now )
 {
     float ret = 0.0f;
 
     if( r )
-        ret = rateForInterval( r, LONG_INTERVAL_MSEC );
+        ret = rateForInterval( r, TR_RC_HISTORY_MSEC, now );
 
     return ret;
 }
@@ -123,34 +73,17 @@ tr_rcRate( const tr_ratecontrol * r )
 ***/
 
 void
-tr_rcTransferred( tr_ratecontrol * r, size_t size )
+tr_rcTransferred( tr_ratecontrol * r,
+                  size_t           size )
 {
-    const uint64_t now = tr_date ();
+    const uint64_t now = tr_date ( );
 
-    if( r->transfers[r->newest].date + GRANULARITY_MSEC >= now )
+    if( r->transfers[r->newest].date + TR_RC_GRANULARITY_MSEC >= now )
         r->transfers[r->newest].size += size;
-    else {
-        if( ++r->newest == HISTORY_SIZE ) r->newest = 0;
+    else
+    {
+        if( ++r->newest == TR_RC_HISTORY_SIZE ) r->newest = 0;
         r->transfers[r->newest].date = now;
         r->transfers[r->newest].size = size;
     }
 }
-
-void
-tr_rcReset( tr_ratecontrol * r )
-{
-    r->newest = 0;
-    memset( r->transfers, 0, sizeof(struct tr_transfer) * HISTORY_SIZE );
-}
-
-void
-tr_rcSetLimit( tr_ratecontrol * r, int limit )
-{
-    r->limit = limit;
-}
-
-int
-tr_rcGetLimit( const tr_ratecontrol * r )
-{
-    return r->limit;
-}
