@@ -2,21 +2,32 @@
 #include <stdlib.h>
 #include <string.h>
 #include "transmission.h"
+#include "session.h"
 #include "utils.h"
 
-#define VERBOSE 0
+#undef VERBOSE
 
-#define check(A) { \
-    ++test; \
-    if (A) { \
-        if( VERBOSE ) \
+#ifdef VERBOSE
+  #define check( A ) \
+    { \
+        ++test; \
+        if( A ){ \
             fprintf( stderr, "PASS test #%d (%s, %d)\n", test, __FILE__, __LINE__ ); \
-    } else { \
-        if( VERBOSE ) \
+        } else { \
             fprintf( stderr, "FAIL test #%d (%s, %d)\n", test, __FILE__, __LINE__ ); \
-        return test; \
-    } \
-}
+            return test; \
+        } \
+    }
+#else
+  #define check( A ) \
+    { \
+        ++test; \
+        if( !( A ) ){ \
+            fprintf( stderr, "FAIL test #%d (%s, %d)\n", test, __FILE__, __LINE__ ); \
+            return test; \
+        } \
+    }
+#endif
 
 int
 main( void )
@@ -24,16 +35,17 @@ main( void )
     int i;
     int test = 0;
 
-    for( i=0; i<100000; ++i )
+    for( i = 0; i < 100000; ++i )
     {
-        int j;
-        int val = 0;
+        int       j;
+        int       val = 0;
         uint8_t * pch = tr_peerIdNew( );
 
         check( strlen( (char*)pch ) == 20 );
         check( !memcmp( pch, PEERID_PREFIX, 8 ) );
 
-        for( j=8; j<20; ++j ) {
+        for( j = 8; j < 20; ++j )
+        {
             char tmp[2] = { pch[j], '\0' };
             val += strtoul( tmp, NULL, 36 );
         }
@@ -45,3 +57,4 @@ main( void )
 
     return 0;
 }
+
