@@ -1,15 +1,16 @@
 /*
- * This file Copyright (C) 2007-2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license. 
+ * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: verify.c 6425 2008-08-01 16:43:22Z charles $
+ * $Id: verify.c 7813 2009-01-29 18:18:24Z charles $
  */
 
+#include <unistd.h> /* S_ISREG */
 #include <sys/stat.h>
 
 #include "transmission.h"
@@ -28,50 +29,54 @@
 
 struct verify_node
 {
-    tr_torrent * torrent;
-    tr_verify_done_cb verify_done_cb;
+    tr_torrent *         torrent;
+    tr_verify_done_cb    verify_done_cb;
 };
 
 static void
-fireCheckDone( tr_torrent          * torrent,
-               tr_verify_done_cb     verify_done_cb )
+fireCheckDone( tr_torrent * tor, tr_verify_done_cb verify_done_cb )
 {
+    assert( tr_isTorrent( tor ) );
+
     if( verify_done_cb )
-        verify_done_cb( torrent );
+        verify_done_cb( tor );
 }
 
 static struct verify_node currentNode;
-
 static tr_list * verifyList = NULL;
-
 static tr_thread * verifyThread = NULL;
-
 static int stopCurrent = FALSE;
 
-static tr_lock* getVerifyLock( void )
+static tr_lock*
+getVerifyLock( void )
 {
     static tr_lock * lock = NULL;
+
     if( lock == NULL )
         lock = tr_lockNew( );
     return lock;
 }
 
 static int
-checkFile( tr_torrent       * tor,
-           tr_file_index_t    fileIndex,
-           int              * abortFlag )
+checkFile( tr_torrent      * tor,
+           void            * buffer,
+           size_t            buflen,
+           tr_file_index_t   fileIndex,
+           int             * abortFlag )
 {
     tr_piece_index_t i;
-    int changed = FALSE;
-    int nofile;
-    struct stat sb;
-    char path[MAX_PATH_LENGTH];
-    const tr_file * file = &tor->info.files[fileIndex];
+    int              changed = FALSE;
+    int              nofile;
+    struct stat      sb;
+    char           * path;
+    const tr_file  * file = &tor->info.files[fileIndex];
 
-    tr_buildPath ( path, sizeof(path), tor->downloadDir, file->name, NULL );
+    path = tr_buildPath( tor->downloadDir, file->name, NULL );
     nofile = stat( path, &sb ) || !S_ISREG( sb.st_mode );
 
-    for( i=file->firstPiece; i<=file->lastPiece && i<tor->info.pieceCount && (!*abortFlag); ++i )
+    for( i = file->firstPiece;
+         i <= file->lastPiece && i < tor->info.pieceCount && ( !*abortFlag );
+         ++i )
     {
         if( nofile )
         {
@@ -79,10 +84,9 @@ checkFile( tr_torrent       * tor,
         }
         else if( !tr_torrentIsPieceChecked( tor, i ) )
         {
-            const int wasComplete = tr_cpPieceIsComplete( tor->completion, i );
-            const tr_errno err = tr_ioTestPiece( tor, i );
+            const int wasComplete = tr_cpPieceIsComplete( &tor->completion, i );
 
-            if( !err ) /* yay */
+            if( tr_ioTestPiece( tor, i, buffer, buflen ) ) /* yay */
             {
                 tr_torrentSetHasPiece( tor, i, TRUE );
                 if( !wasComplete )
@@ -95,7 +99,8 @@ checkFile( tr_torrent       * tor,
                  * it being incomplete, do nothing -- we don't
                  * want to lose blocks in those incomplete pieces */
 
-                if( wasComplete ) {
+                if( wasComplete )
+                {
                     tr_torrentSetHasPiece( tor, i, FALSE );
                     changed = TRUE;
                 }
@@ -105,6 +110,8 @@ checkFile( tr_torrent       * tor,
         tr_torrentSetPieceChecked( tor, i, TRUE );
     }
 
+    tr_free( path );
+
     return changed;
 }
 
@@ -113,15 +120,17 @@ verifyThreadFunc( void * unused UNUSED )
 {
     for( ;; )
     {
-        int changed = 0;
-        tr_file_index_t i;
-        tr_torrent * tor;
+        int                  changed = 0;
+        tr_file_index_t      i;
+        tr_torrent         * tor;
         struct verify_node * node;
+        void               * buffer;
 
         tr_lockLock( getVerifyLock( ) );
         stopCurrent = FALSE;
         node = (struct verify_node*) verifyList ? verifyList->data : NULL;
-        if( node == NULL ) {
+        if( node == NULL )
+        {
             currentNode.torrent = NULL;
             break;
         }
@@ -132,13 +141,15 @@ verifyThreadFunc( void * unused UNUSED )
         tr_free( node );
         tr_lockUnlock( getVerifyLock( ) );
 
-        tor->verifyState = TR_VERIFY_NOW;
-
         tr_torinf( tor, _( "Verifying torrent" ) );
-        for( i=0; i<tor->info.fileCount && !stopCurrent; ++i )
-            changed |= checkFile( tor, i, &stopCurrent );
-
+        assert( tr_isTorrent( tor ) );
+        tor->verifyState = TR_VERIFY_NOW;
+        buffer = tr_new( uint8_t, tor->info.pieceSize );
+        for( i = 0; i < tor->info.fileCount && !stopCurrent; ++i )
+            changed |= checkFile( tor, buffer, tor->info.pieceSize, i, &stopCurrent );
+        tr_free( buffer );
         tor->verifyState = TR_VERIFY_NONE;
+        assert( tr_isTorrent( tor ) );
 
         if( !stopCurrent )
         {
@@ -153,11 +164,13 @@ verifyThreadFunc( void * unused UNUSED )
 }
 
 void
-tr_verifyAdd( tr_torrent          * tor,
-              tr_verify_done_cb    verify_done_cb )
+tr_verifyAdd( tr_torrent *      tor,
+              tr_verify_done_cb verify_done_cb )
 {
     const int uncheckedCount = tr_torrentCountUncheckedPieces( tor );
 
+    assert( tr_isTorrent( tor ) );
+
     if( !uncheckedCount )
     {
         /* doesn't need to be checked... */
@@ -177,25 +190,45 @@ tr_verifyAdd( tr_torrent          * tor,
         tor->verifyState = verifyList ? TR_VERIFY_WAIT : TR_VERIFY_NOW;
         tr_list_append( &verifyList, node );
         if( verifyThread == NULL )
-            verifyThread = tr_threadNew( verifyThreadFunc, NULL, "verifyThreadFunc" );
+            verifyThread = tr_threadNew( verifyThreadFunc, NULL );
         tr_lockUnlock( getVerifyLock( ) );
     }
 }
 
 static int
-compareVerifyByTorrent( const void * va, const void * vb )
+compareVerifyByTorrent( const void * va,
+                        const void * vb )
 {
     const struct verify_node * a = va;
-    const tr_torrent * b = vb;
+    const tr_torrent *         b = vb;
+
     return a->torrent - b;
 }
 
+int
+tr_verifyInProgress( const tr_torrent * tor )
+{
+    int found = FALSE;
+    tr_lock * lock = getVerifyLock( );
+    tr_lockLock( lock );
+
+    assert( tr_isTorrent( tor ) );
+
+    found = ( tor == currentNode.torrent )
+         || ( tr_list_find( verifyList, tor, compareVerifyByTorrent ) != NULL );
+
+    tr_lockUnlock( lock );
+    return found;
+}
+
 void
 tr_verifyRemove( tr_torrent * tor )
 {
     tr_lock * lock = getVerifyLock( );
     tr_lockLock( lock );
 
+    assert( tr_isTorrent( tor ) );
+
     if( tor == currentNode.torrent )
     {
         stopCurrent = TRUE;
@@ -214,3 +247,4 @@ tr_verifyRemove( tr_torrent * tor )
 
     tr_lockUnlock( lock );
 }
+
