@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: CreatorWindowController.h 5710 2008-04-27 14:31:49Z livings124 $
+ * $Id: CreatorWindowController.h 7666 2009-01-11 18:04:45Z livings124 $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -29,15 +29,18 @@
 @interface CreatorWindowController : NSWindowController
 {
     IBOutlet NSImageView * fIconView;
-    IBOutlet NSTextField * fNameField, * fStatusField, * fPiecesField, * fTrackerField, * fLocationField;
+    IBOutlet NSTextField * fNameField, * fStatusField, * fPiecesField, * fLocationField;
+    IBOutlet NSTableView * fTrackerTable;
+    IBOutlet NSSegmentedControl * fTrackerAddRemoveControl;
     IBOutlet NSTextView * fCommentView;
-    IBOutlet NSButton *fCreateButton, * fPrivateCheck, * fOpenCheck;
+    IBOutlet NSButton * fPrivateCheck, * fOpenCheck;
     
     IBOutlet NSView * fProgressView;
     IBOutlet NSProgressIndicator * fProgressIndicator;
     
     tr_metainfo_builder * fInfo;
     NSString * fPath, * fLocation;
+    NSMutableArray * fTrackers;
     BOOL fOpenTorrent;
     
     NSTimer * fTimer;
@@ -46,14 +49,18 @@
     NSUserDefaults * fDefaults;
 }
 
-+ (void) createTorrentFile: (tr_handle *) handle;
-+ (void) createTorrentFile: (tr_handle *) handle forFile: (NSString *) file;
++ (void) createTorrentFile: (tr_session *) handle;
++ (void) createTorrentFile: (tr_session *) handle forFile: (NSString *) file;
 
-- (id) initWithHandle: (tr_handle *) handle path: (NSString *) path;
+- (id) initWithHandle: (tr_session *) handle path: (NSString *) path;
+
+- (void) toggleOpenCheck: (id) sender;
 
 - (void) setLocation: (id) sender;
 - (void) create: (id) sender;
 - (void) cancelCreateWindow: (id) sender;
 - (void) cancelCreateProgress: (id) sender;
 
+- (void) addRemoveTracker: (id) sender;
+
 @end
