@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: CreatorWindowController.m 5718 2008-04-28 18:51:38Z livings124 $
+ * $Id: CreatorWindowController.m 7780 2009-01-23 02:39:42Z livings124 $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -23,22 +23,29 @@
  *****************************************************************************/
 
 #import "CreatorWindowController.h"
+#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
-#include "utils.h" //tr_httpParseURL
+#import "utils.h" //tr_httpIsValidURL
+
+#define TRACKER_ADD_TAG 0
+#define TRACKER_REMOVE_TAG 1
 
 @interface CreatorWindowController (Private)
 
 + (NSString *) chooseFile;
-- (void) updateEnableCreateButtonForTrackerField;
-- (void) locationSheetClosed: (NSSavePanel *) openPanel returnCode: (int) code contextInfo: (void *) info;
+- (void) updateEnableOpenCheckForTrackers;
+- (void) locationSheetClosed: (NSSavePanel *) openPanel returnCode: (NSInteger) code contextInfo: (void *) info;
+
+- (void) createBlankAddressAlertDidEnd: (NSAlert *) alert returnCode: (NSInteger) returnCode contextInfo: (void *) contextInfo;
+- (void) createReal;
 - (void) checkProgress;
-- (void) failureSheetClosed: (NSAlert *) alert returnCode: (int) code contextInfo: (void *) info;
+- (void) failureSheetClosed: (NSAlert *) alert returnCode: (NSInteger) code contextInfo: (void *) info;
 
 @end
 
 @implementation CreatorWindowController
 
-+ (void) createTorrentFile: (tr_handle *) handle
++ (void) createTorrentFile: (tr_session *) handle
 {
     //get file/folder for torrent
     NSString * path;
@@ -49,13 +56,13 @@
     [creator showWindow: nil];
 }
 
-+ (void) createTorrentFile: (tr_handle *) handle forFile: (NSString *) file
++ (void) createTorrentFile: (tr_session *) handle forFile: (NSString *) file
 {
     CreatorWindowController * creator = [[self alloc] initWithHandle: handle path: file];
     [creator showWindow: nil];
 }
 
-- (id) initWithHandle: (tr_handle *) handle path: (NSString *) path
+- (id) initWithHandle: (tr_session *) handle path: (NSString *) path
 {
     if ((self = [super initWithWindowNibName: @"Creator"]))
     {
@@ -98,6 +105,31 @@
         }
         
         fDefaults = [NSUserDefaults standardUserDefaults];
+        
+        //get list of trackers
+        if (!(fTrackers = [[fDefaults arrayForKey: @"CreatorTrackers"] mutableCopy]))
+        {
+            fTrackers = [[NSMutableArray alloc] initWithCapacity: 1];
+            
+            //check for single tracker from versions before 1.3
+            NSString * tracker;
+            if ((tracker = [fDefaults stringForKey: @"CreatorTracker"]))
+            {
+                [fDefaults removeObjectForKey: @"CreatorTracker"];
+                if (![tracker isEqualToString: @""])
+                {
+                    [fTrackers addObject: tracker];
+                    [fDefaults setObject: fTrackers forKey: @"CreatorTrackers"];
+                }
+            }
+        }
+        
+        //remove potentially invalid addresses
+        for (NSInteger i = [fTrackers count]-1; i >= 0; i--)
+        {
+            if (!tr_httpIsValidURL([[fTrackers objectAtIndex: i] UTF8String]))
+                [fTrackers removeObjectAtIndex: i];
+        }
     }
     return self;
 }
@@ -122,7 +154,7 @@
     if (multifile)
     {
         NSString * fileString;
-        int count = fInfo->fileCount;
+        NSInteger count = fInfo->fileCount;
         if (count != 1)
             fileString = [NSString stringWithFormat: NSLocalizedString(@"%d files", "Create torrent -> info"), count];
         else
@@ -147,16 +179,18 @@
     [fLocationField setToolTip: fLocation];
     
     //set previously saved values
-    NSString * tracker;
-    if ((tracker = [fDefaults stringForKey: @"CreatorTracker"]))
-        [fTrackerField setStringValue: tracker];
-    [self updateEnableCreateButtonForTrackerField];
-    
     if ([fDefaults objectForKey: @"CreatorPrivate"])
         [fPrivateCheck setState: [fDefaults boolForKey: @"CreatorPrivate"] ? NSOnState : NSOffState];
     
-    if ([fDefaults objectForKey: @"CreatorOpen"])
-        [fOpenCheck setState: [fDefaults boolForKey: @"CreatorOpen"] ? NSOnState : NSOffState];
+    fOpenTorrent = [fDefaults boolForKey: @"CreatorOpen"];
+    [self updateEnableOpenCheckForTrackers];
+    
+    if (![NSApp isOnLeopardOrBetter])
+    {
+        [fTrackerAddRemoveControl sizeToFit];
+        [fTrackerAddRemoveControl setLabel: @"+" forSegment: TRACKER_ADD_TAG];
+        [fTrackerAddRemoveControl setLabel: @"-" forSegment: TRACKER_REMOVE_TAG];
+    }
 }
 
 - (void) dealloc
@@ -164,15 +198,21 @@
     [fPath release];
     [fLocation release];
     
+    [fTrackers release];
+    
     if (fInfo)
         tr_metaInfoBuilderFree(fInfo);
     
-    if (fTimer)
-        [fTimer invalidate];
+    [fTimer invalidate];
     
     [super dealloc];
 }
 
+- (void) toggleOpenCheck: (id) sender
+{
+    fOpenTorrent = [fOpenCheck state] == NSOnState;
+}
+
 - (void) setLocation: (id) sender
 {
     NSSavePanel * panel = [NSSavePanel savePanel];
@@ -189,74 +229,28 @@
             contextInfo: nil];
 }
 
-- (void) controlTextDidChange: (NSNotification *) notification
-{
-    if ([notification object] == fTrackerField)
-        [self updateEnableCreateButtonForTrackerField];
-}
-
 - (void) create: (id) sender
 {
-    NSString * trackerString = [fTrackerField stringValue];
-    if ([trackerString rangeOfString: @"://"].location == NSNotFound)
-        trackerString = [@"http://" stringByAppendingString: trackerString];
-    
-    //parse tracker string
-    if (!tr_httpIsValidURL([trackerString UTF8String]))
+    if ([fTrackers count] == 0 && [fDefaults boolForKey: @"WarningCreatorBlankAddress"])
     {
-        NSAlert * alert = [[[NSAlert alloc] init] autorelease];
-        [alert addButtonWithTitle: NSLocalizedString(@"OK", "Create torrent -> warning -> button")];
-        [alert setInformativeText: NSLocalizedString(@"Change the tracker address to create the torrent file.",
-                                                    "Create torrent -> warning -> info")];
-        [alert setAlertStyle: NSWarningAlertStyle];
+        NSAlert * alert = [[NSAlert alloc] init];
+        [alert setMessageText: NSLocalizedString(@"There is no tracker address.", "Create torrent -> blank address -> title")];
+        [alert setInformativeText: NSLocalizedString(@"The torrent file will not be able to be opened."
+            " A torrent file with no tracker address is only useful when you plan to upload the file to a tracker website"
+            " that will add the address for you.", "Create torrent -> blank address -> message")];
+        [alert addButtonWithTitle: NSLocalizedString(@"Create", "Create torrent -> blank address -> button")];
+        [alert addButtonWithTitle: NSLocalizedString(@"Cancel", "Create torrent -> blank address -> button")];
         
-        //check common reasons for failure
-        NSString * prefix = [trackerString substringToIndex: [trackerString rangeOfString: @"://"].location];
-        if ([prefix caseInsensitiveCompare: @"http"] != NSOrderedSame && [prefix caseInsensitiveCompare: @"https"] != NSOrderedSame)
-            [alert setMessageText: NSLocalizedString(@"The tracker address must begin with \"http://\" or \"https://\".",
-                                                    "Create torrent -> warning -> message")];
+        if ([NSApp isOnLeopardOrBetter])
+            [alert setShowsSuppressionButton: YES];
         else
-            [alert setMessageText: NSLocalizedString(@"The tracker address is invalid.", "Create torrent -> warning -> message")];
-        
-        [alert beginSheetModalForWindow: [self window] modalDelegate: self didEndSelector: nil contextInfo: nil];
-        return;
+            [alert addButtonWithTitle: NSLocalizedString(@"Don't Alert Again", "Create torrent -> blank address -> button")];
+
+        [alert beginSheetModalForWindow: [self window] modalDelegate: self
+            didEndSelector: @selector(createBlankAddressAlertDidEnd:returnCode:contextInfo:) contextInfo: nil];
     }
-    
-    //check if a file with the same name and location already exists
-    if ([[NSFileManager defaultManager] fileExistsAtPath: fLocation])
-    {
-        NSArray * pathComponents = [fLocation pathComponents];
-        int count = [pathComponents count];
-        
-        NSAlert * alert = [[[NSAlert alloc] init] autorelease];
-        [alert addButtonWithTitle: NSLocalizedString(@"OK", "Create torrent -> file already exists warning -> button")];
-        [alert setMessageText: NSLocalizedString(@"A torrent file with this name and directory cannot be created.",
-                                                "Create torrent -> file already exists warning -> title")];
-        [alert setInformativeText: [NSString stringWithFormat:
-                NSLocalizedString(@"A file with the name \"%@\" already exists in the directory \"%@\". "
-                    "Choose a new name or directory to create the torrent file.",
-                    "Create torrent -> file already exists warning -> warning"),
-                    [pathComponents objectAtIndex: count-1], [pathComponents objectAtIndex: count-2]]];
-        [alert setAlertStyle: NSWarningAlertStyle];
-        
-        [alert beginSheetModalForWindow: [self window] modalDelegate: self didEndSelector: nil contextInfo: nil];
-        return;
-    }
-    
-    fOpenTorrent = [fOpenCheck state] == NSOnState;
-    
-    //store values
-    [fDefaults setObject: trackerString forKey: @"CreatorTracker"];
-    [fDefaults setBool: [fPrivateCheck state] == NSOnState forKey: @"CreatorPrivate"];
-    [fDefaults setBool: fOpenTorrent forKey: @"CreatorOpen"];
-    [fDefaults setObject: [fLocation stringByDeletingLastPathComponent] forKey: @"CreatorLocation"];
-    
-    [[NSNotificationCenter defaultCenter] postNotificationName: @"BeginCreateTorrentFile" object: fLocation userInfo: nil];
-    tr_makeMetaInfo(fInfo, [fLocation UTF8String], [trackerString UTF8String], [[fCommentView string] UTF8String],
-                    [fPrivateCheck state] == NSOnState);
-    
-    fTimer = [NSTimer scheduledTimerWithTimeInterval: 0.1 target: self selector: @selector(checkProgress)
-                        userInfo: nil repeats: YES];
+    else
+        [self createReal];
 }
 
 - (void) cancelCreateWindow: (id) sender
@@ -275,6 +269,72 @@
     [fTimer fire];
 }
 
+- (NSInteger) numberOfRowsInTableView: (NSTableView *) tableView
+{
+    return [fTrackers count];
+}
+
+- (id) tableView: (NSTableView *) tableView objectValueForTableColumn: (NSTableColumn *) tableColumn row: (NSInteger) row
+{
+    return [fTrackers objectAtIndex: row];
+}
+
+- (void) addRemoveTracker: (id) sender
+{
+    //don't allow add/remove when currently adding - it leads to weird results
+    if ([fTrackerTable editedRow] != -1)
+        return;
+    
+    if ([[sender cell] tagForSegment: [sender selectedSegment]] == TRACKER_REMOVE_TAG)
+    {
+        [fTrackers removeObjectsAtIndexes: [fTrackerTable selectedRowIndexes]];
+        
+        [fTrackerTable deselectAll: self];
+        [fTrackerTable reloadData];
+        
+        [self updateEnableOpenCheckForTrackers];
+    }
+    else
+    {
+        [fTrackers addObject: @""];
+        [fTrackerTable reloadData];
+        
+        NSInteger row = [fTrackers count] - 1;
+        [fTrackerTable selectRow: row byExtendingSelection: NO];
+        [fTrackerTable editColumn: 0 row: row withEvent: nil select: YES];
+    }
+}
+
+- (void) tableView: (NSTableView *) tableView setObjectValue: (id) object forTableColumn: (NSTableColumn *) tableColumn
+    row: (NSInteger) row
+{
+    NSString * tracker = (NSString *)object;
+    
+    tracker = [tracker stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
+    
+    if ([tracker rangeOfString: @"://"].location == NSNotFound)
+        tracker = [@"http://" stringByAppendingString: tracker];
+    
+    if (!tr_httpIsValidURL([tracker UTF8String]))
+    {
+        NSBeep();
+        [fTrackers removeObjectAtIndex: row];
+    }
+    else
+    {
+        [fTrackers replaceObjectAtIndex: row withObject: tracker];
+        [self updateEnableOpenCheckForTrackers];
+    }
+    
+    [fTrackerTable deselectAll: self];
+    [fTrackerTable reloadData];
+}
+
+- (void) tableViewSelectionDidChange: (NSNotification *) notification
+{
+    [fTrackerAddRemoveControl setEnabled: [fTrackerTable numberOfSelectedRows] > 0 forSegment: TRACKER_REMOVE_TAG];
+}
+
 @end
 
 @implementation CreatorWindowController (Private)
@@ -296,23 +356,14 @@
     return success ? [[panel filenames] objectAtIndex: 0] : nil;
 }
 
-- (void) updateEnableCreateButtonForTrackerField
+- (void) updateEnableOpenCheckForTrackers
 {
-    NSString * string = [fTrackerField stringValue];
-    BOOL enable = YES;
-    if ([string isEqualToString: @""])
-        enable = NO;
-    else
-    {
-        NSRange prefixRange = [string rangeOfString: @"://"];
-        if (prefixRange.location != NSNotFound && [string length] == NSMaxRange(prefixRange))
-            enable = NO;
-    }
-    
-    [fCreateButton setEnabled: enable];
+    BOOL hasTracker = [fTrackers count] > 0;
+    [fOpenCheck setEnabled: hasTracker];
+    [fOpenCheck setState: (fOpenTorrent && hasTracker) ? NSOnState : NSOffState];
 }
 
-- (void) locationSheetClosed: (NSSavePanel *) panel returnCode: (int) code contextInfo: (void *) info
+- (void) locationSheetClosed: (NSSavePanel *) panel returnCode: (NSInteger) code contextInfo: (void *) info
 {
     if (code == NSOKButton)
     {
@@ -324,6 +375,61 @@
     }
 }
 
+- (void) createBlankAddressAlertDidEnd: (NSAlert *) alert returnCode: (NSInteger) returnCode contextInfo: (void *) contextInfo
+{
+    if (([NSApp isOnLeopardOrBetter] ? [[alert suppressionButton] state] == NSOnState : returnCode == NSAlertThirdButtonReturn))
+        [[NSUserDefaults standardUserDefaults] setBool: NO forKey: @"WarningCreatorBlankAddress"];
+    
+    [alert release];
+    
+    if (returnCode == NSAlertFirstButtonReturn)
+        [self performSelectorOnMainThread: @selector(createReal) withObject: nil waitUntilDone: NO];
+}
+
+- (void) createReal
+{
+    //check if a file with the same name and location already exists
+    if ([[NSFileManager defaultManager] fileExistsAtPath: fLocation])
+    {
+        NSArray * pathComponents = [fLocation pathComponents];
+        NSInteger count = [pathComponents count];
+        
+        NSAlert * alert = [[[NSAlert alloc] init] autorelease];
+        [alert addButtonWithTitle: NSLocalizedString(@"OK", "Create torrent -> file already exists warning -> button")];
+        [alert setMessageText: NSLocalizedString(@"A torrent file with this name and directory cannot be created.",
+                                                "Create torrent -> file already exists warning -> title")];
+        [alert setInformativeText: [NSString stringWithFormat:
+                NSLocalizedString(@"A file with the name \"%@\" already exists in the directory \"%@\". "
+                    "Choose a new name or directory to create the torrent file.",
+                    "Create torrent -> file already exists warning -> warning"),
+                    [pathComponents objectAtIndex: count-1], [pathComponents objectAtIndex: count-2]]];
+        [alert setAlertStyle: NSWarningAlertStyle];
+        
+        [alert beginSheetModalForWindow: [self window] modalDelegate: self didEndSelector: nil contextInfo: nil];
+        return;
+    }
+    
+    //parse non-empty tracker strings
+    tr_tracker_info * trackerInfo = tr_new0(tr_tracker_info, [fTrackers count]);
+    
+    for (NSUInteger i = 0; i < [fTrackers count]; i++)
+        trackerInfo[i].announce = (char *)[[fTrackers objectAtIndex: i] UTF8String];
+    
+    //store values
+    [fDefaults setObject: fTrackers forKey: @"CreatorTrackers"];
+    [fDefaults setBool: [fPrivateCheck state] == NSOnState forKey: @"CreatorPrivate"];
+    [fDefaults setBool: fOpenTorrent forKey: @"CreatorOpen"];
+    [fDefaults setObject: [fLocation stringByDeletingLastPathComponent] forKey: @"CreatorLocation"];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"BeginCreateTorrentFile" object: fLocation userInfo: nil];
+    tr_makeMetaInfo(fInfo, [fLocation UTF8String], trackerInfo, [fTrackers count], [[fCommentView string] UTF8String],
+                    [fPrivateCheck state] == NSOnState);
+    tr_free(trackerInfo);
+    
+    fTimer = [NSTimer scheduledTimerWithTimeInterval: 0.1 target: self selector: @selector(checkProgress)
+                userInfo: nil repeats: YES];
+}
+
 - (void) checkProgress
 {
     if (fInfo->isDone)
@@ -335,7 +441,7 @@
         switch (fInfo->result)
         {
             case TR_MAKEMETA_OK:
-                if (fOpenTorrent)
+                if (fOpenTorrent && [fTrackers count] > 0)
                 {
                     NSDictionary * dict = [[NSDictionary alloc] initWithObjectsAndKeys: fLocation, @"File",
                                             [fPath stringByDeletingLastPathComponent], @"Path", nil];
@@ -362,7 +468,9 @@
                 else if (fInfo->result == TR_MAKEMETA_IO_WRITE)
                     [alert setInformativeText: [NSString stringWithFormat: NSLocalizedString(@"Could not write \"%s\": %s.",
                         "Create torrent -> failed -> warning"), fInfo->errfile, strerror(fInfo->my_errno)]];
-                else; //invalid url should have been caught before creating
+                else //invalid url should have been caught before creating
+                    [alert setInformativeText: [NSString stringWithFormat: @"%@ (%d)",
+                        NSLocalizedString(@"An unknown error has occurred.", "Create torrent -> failed -> warning"), fInfo->result]];
                 
                 [alert beginSheetModalForWindow: [self window] modalDelegate: self
                         didEndSelector: @selector(failureSheetClosed:returnCode:contextInfo:) contextInfo: nil];
@@ -376,15 +484,17 @@
         {
             fStarted = YES;
             
+            [fProgressView setHidden: YES];
+            
             NSWindow * window = [self window];
             
             NSRect windowRect = [window frame];
-            float difference = [fProgressView frame].size.height - [[window contentView] frame].size.height;
+            CGFloat difference = [fProgressView frame].size.height - [[window contentView] frame].size.height;
             windowRect.origin.y -= difference;
             windowRect.size.height += difference;
             
             //don't allow vertical resizing
-            float height = windowRect.size.height;
+            CGFloat height = windowRect.size.height;
             [window setMinSize: NSMakeSize([window minSize].width, height)];
             [window setMaxSize: NSMakeSize([window maxSize].width, height)];
             
@@ -397,7 +507,7 @@
     }
 }
 
-- (void) failureSheetClosed: (NSAlert *) alert returnCode: (int) code contextInfo: (void *) info
+- (void) failureSheetClosed: (NSAlert *) alert returnCode: (NSInteger) code contextInfo: (void *) info
 {
     [[alert window] orderOut: nil];
     [[self window] close];
