@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: FileNameCell.m 5581 2008-04-10 19:52:27Z livings124 $
+ * $Id: FileNameCell.m 7012 2008-11-01 17:52:25Z livings124 $
  * 
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -25,20 +25,22 @@
 #import "FileNameCell.h"
 #import "FileOutlineView.h"
 #import "Torrent.h"
+#import "FileListNode.h"
 #import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 
-#define PADDING_HORIZONAL 2.0
-#define IMAGE_FOLDER_SIZE 16.0
-#define IMAGE_ICON_SIZE 32.0
-#define PADDING_BETWEEN_IMAGE_AND_TITLE 4.0
-#define PADDING_ABOVE_TITLE_FILE 2.0
-#define PADDING_BELOW_STATUS_FILE 2.0
+#define PADDING_HORIZONAL 2.0f
+#define IMAGE_FOLDER_SIZE 16.0f
+#define IMAGE_ICON_SIZE 32.0f
+#define PADDING_BETWEEN_IMAGE_AND_TITLE 4.0f
+#define PADDING_ABOVE_TITLE_FILE 2.0f
+#define PADDING_BELOW_STATUS_FILE 2.0f
+#define PADDING_BETWEEN_NAME_AND_FOLDER_STATUS 4.0f
 
 @interface FileNameCell (Private)
 
 - (NSRect) rectForTitleWithString: (NSAttributedString *) string inBounds: (NSRect) bounds;
-- (NSRect) rectForStatusWithString: (NSAttributedString *) string inBounds: (NSRect) bounds;
+- (NSRect) rectForStatusWithString: (NSAttributedString *) string withTitleRect: (NSRect) titleRect inBounds: (NSRect) bounds;
 
 - (NSAttributedString *) attributedTitleWithColor: (NSColor *) color;
 - (NSAttributedString *) attributedStatusWithColor: (NSColor *) color;
@@ -55,11 +57,11 @@
         [paragraphStyle setLineBreakMode: NSLineBreakByTruncatingTail];
         
         fTitleAttributes = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
-                            [NSFont messageFontOfSize: 12.0], NSFontAttributeName,
+                            [NSFont messageFontOfSize: 12.0f], NSFontAttributeName,
                             paragraphStyle, NSParagraphStyleAttributeName, nil];
         
         fStatusAttributes = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
-                                [NSFont messageFontOfSize: 9.0], NSFontAttributeName,
+                                [NSFont messageFontOfSize: 9.0f], NSFontAttributeName,
                                 paragraphStyle, NSParagraphStyleAttributeName, nil];
         
         [paragraphStyle release];
@@ -91,8 +93,9 @@
 
 - (NSImage *) image
 {
-    NSImage * image = [[self objectValue] objectForKey: @"Icon"];
-    if (!image)
+    FileListNode * node = (FileListNode *)[self objectValue];
+    NSImage * image;
+    if ([node isFolder])
     {
         if (!fFolderImage)
         {
@@ -101,6 +104,9 @@
         }
         image = fFolderImage;
     }
+    else
+        image = [node icon];
+    
     return image;
 }
 
@@ -110,8 +116,8 @@
     
     result.origin.x += PADDING_HORIZONAL;
     
-    const float IMAGE_SIZE = [[[self objectValue] objectForKey: @"IsFolder"] boolValue] ? IMAGE_FOLDER_SIZE : IMAGE_ICON_SIZE;
-    result.origin.y += (result.size.height - IMAGE_SIZE) * 0.5;
+    const CGFloat IMAGE_SIZE = [(FileListNode *)[self objectValue] isFolder] ? IMAGE_FOLDER_SIZE : IMAGE_ICON_SIZE;
+    result.origin.y += (result.size.height - IMAGE_SIZE) * 0.5f;
     result.size = NSMakeSize(IMAGE_SIZE, IMAGE_SIZE);
     
     return result;
@@ -124,7 +130,9 @@
 
 - (NSRect) statusRectForBounds: (NSRect) bounds
 {
-    return [self rectForStatusWithString: [self attributedStatusWithColor: nil] inBounds: bounds];
+    return [self rectForStatusWithString: [self attributedStatusWithColor: nil]
+            withTitleRect: [(FileListNode *)[self objectValue] isFolder] ? [self titleRectForBounds: bounds] : NSZeroRect
+            inBounds: bounds];
 }
 
 - (void) drawWithFrame: (NSRect) cellFrame inView: (NSView *) controlView
@@ -137,8 +145,7 @@
     if ([self isHighlighted]
             && [[self highlightColorWithFrame: cellFrame inView: controlView] isEqual: [NSColor alternateSelectedControlColor]])
         specialColor = [NSColor whiteColor];
-    else if ([[(FileOutlineView *)[self controlView] torrent] checkForFiles:
-                    [[self objectValue] objectForKey: @"Indexes"]] == NSOffState)
+    else if ([[(FileOutlineView *)[self controlView] torrent] checkForFiles: [(FileListNode *)[self objectValue] indexes]] == NSOffState)
         specialColor = [NSColor disabledControlTextColor];
     else;
     
@@ -147,12 +154,9 @@
     [titleString drawInRect: titleRect];
     
     //status
-    if (![[[self objectValue] objectForKey: @"IsFolder"] boolValue])
-    {
-        NSAttributedString * statusString = [self attributedStatusWithColor: specialColor ? specialColor : [NSColor darkGrayColor]];
-        NSRect statusRect = [self rectForStatusWithString: statusString inBounds: cellFrame];
-        [statusString drawInRect: statusRect];
-    }
+    NSAttributedString * statusString = [self attributedStatusWithColor: specialColor ? specialColor : [NSColor darkGrayColor]];
+    NSRect statusRect = [self rectForStatusWithString: statusString withTitleRect: titleRect inBounds: cellFrame];
+    [statusString drawInRect: statusRect];
 }
 
 @end
@@ -165,7 +169,7 @@
     
     NSRect result = bounds;
     
-    if (![[[self objectValue] objectForKey: @"IsFolder"] boolValue])
+    if (![(FileListNode *)[self objectValue] isFolder])
     {
         result.origin.x += PADDING_HORIZONAL + IMAGE_ICON_SIZE + PADDING_BETWEEN_IMAGE_AND_TITLE;
         result.origin.y += PADDING_ABOVE_TITLE_FILE;
@@ -173,7 +177,7 @@
     else
     {
         result.origin.x += PADDING_HORIZONAL + IMAGE_FOLDER_SIZE + PADDING_BETWEEN_IMAGE_AND_TITLE;
-        result.origin.y += (result.size.height - titleSize.height) * 0.5;
+        result.origin.y += (result.size.height - titleSize.height) * 0.5f;
     }
     result.size = titleSize;
     result.size.width = MIN(result.size.width, NSMaxX(bounds) - result.origin.x - PADDING_HORIZONAL);
@@ -181,18 +185,22 @@
     return result;
 }
 
-- (NSRect) rectForStatusWithString: (NSAttributedString *) string inBounds: (NSRect) bounds
+- (NSRect) rectForStatusWithString: (NSAttributedString *) string withTitleRect: (NSRect) titleRect inBounds: (NSRect) bounds;
 {
-    if ([[[self objectValue] objectForKey: @"IsFolder"] boolValue])
-        return NSZeroRect;
-    
     NSSize statusSize = [string size];
     
-    NSRect result = bounds;
-    
-    result.origin.x += PADDING_HORIZONAL + IMAGE_ICON_SIZE + PADDING_BETWEEN_IMAGE_AND_TITLE;
-    result.origin.y += result.size.height - PADDING_BELOW_STATUS_FILE - statusSize.height;
-    
+    NSRect result;
+    if (![(FileListNode *)[self objectValue] isFolder])
+    {
+        result.origin.x = bounds.origin.x + PADDING_HORIZONAL + IMAGE_ICON_SIZE + PADDING_BETWEEN_IMAGE_AND_TITLE;
+        result.origin.y = NSMaxY(bounds) - PADDING_BELOW_STATUS_FILE - statusSize.height;
+    }
+    else
+    {
+        result.origin.x = NSMaxX(titleRect) + PADDING_BETWEEN_NAME_AND_FOLDER_STATUS;
+        result.origin.y = NSMaxY(titleRect) - statusSize.height - 1.0f;
+    }
+        
     result.size = statusSize;
     result.size.width = MIN(result.size.width, NSMaxX(bounds) - result.origin.x - PADDING_HORIZONAL);
     
@@ -204,7 +212,7 @@
     if (color)
         [fTitleAttributes setObject: color forKey: NSForegroundColorAttributeName];
         
-    NSString * title = [[self objectValue] objectForKey: @"Name"];
+    NSString * title = [(FileListNode *)[self objectValue] name];
     return [[[NSAttributedString alloc] initWithString: title attributes: fTitleAttributes] autorelease];
 }
 
@@ -214,11 +222,15 @@
         [fStatusAttributes setObject: color forKey: NSForegroundColorAttributeName];
     
     Torrent * torrent = [(FileOutlineView *)[self controlView] torrent];
-    float percent = [torrent fileProgress: [[[self objectValue] objectForKey: @"Indexes"] firstIndex]] * 100.0;
+    FileListNode * node = (FileListNode *)[self objectValue];
     
-    NSString * status = [NSString localizedStringWithFormat: NSLocalizedString(@"%.2f%% of %@",
-                            "Inspector -> Files tab -> file status string"), percent,
-                            [NSString stringForFileSize: [[[self objectValue] objectForKey: @"Size"] unsignedLongLongValue]]];
+    NSString * percentString;
+    CGFloat progress = [torrent fileProgress: node];
+    percentString = progress == 1.0f ? @"100%" : [NSString localizedStringWithFormat: @"%.2f%%", progress * 100.0f];
+    
+    
+    NSString * status = [NSString localizedStringWithFormat: NSLocalizedString(@"%@ of %@",
+                            "Inspector -> Files tab -> file status string"), percentString, [NSString stringForFileSize: [node size]]];
     
     return [[[NSAttributedString alloc] initWithString: status attributes: fStatusAttributes] autorelease];
 }
