@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: InfoWindowController.m 6325 2008-07-11 21:48:43Z livings124 $
+ * $Id: InfoWindowController.m 7016 2008-11-02 01:07:01Z livings124 $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -23,13 +23,17 @@
  *****************************************************************************/
 
 #import "InfoWindowController.h"
+#import "Torrent.h"
 #import "InfoTabButtonCell.h"
 #import "FileOutlineView.h"
+#import "FileOutlineController.h"
 #import "FileListNode.h"
 #import "TrackerTableView.h"
+#import "PiecesView.h"
 #import "QuickLookController.h"
 #import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
+#include "utils.h" //tr_getRatio()
 
 #define TAB_INFO_IDENT @"Info"
 #define TAB_ACTIVITY_IDENT @"Activity"
@@ -70,7 +74,7 @@ typedef enum
 - (void) updateInfoPeers;
 - (void) updateInfoFiles;
 
-- (NSView *) tabViewForTag: (int) tag;
+- (NSView *) tabViewForTag: (NSInteger) tag;
 - (void) setWebSeedTableHidden: (BOOL) hide animate: (BOOL) animate;
 - (NSArray *) peerSortDescriptors;
 
@@ -93,7 +97,7 @@ typedef enum
     //window location and size
     NSPanel * window = (NSPanel *)[self window];
     
-    float windowHeight = [window frame].size.height;
+    CGFloat windowHeight = [window frame].size.height;
     
     [window setFrameAutosaveName: @"InspectorWindow"];
     [window setFrameUsingName: @"InspectorWindow"];
@@ -116,7 +120,7 @@ typedef enum
     //set selected tab
     fCurrentTabTag = INVALID;
     NSString * identifier = [[NSUserDefaults standardUserDefaults] stringForKey: @"InspectorSelected"];
-    int tag;
+    NSInteger tag;
     if ([identifier isEqualToString: TAB_INFO_IDENT])
         tag = TAB_INFO_TAG;
     else if ([identifier isEqualToString: TAB_ACTIVITY_IDENT])
@@ -238,7 +242,7 @@ typedef enum
     [fTorrents release];
     fTorrents = [torrents retain];
 
-    int numberSelected = [fTorrents count];
+    NSUInteger numberSelected = [fTorrents count];
     if (numberSelected != 1)
     {
         if (numberSelected > 0)
@@ -250,7 +254,7 @@ typedef enum
                                             "Inspector -> selected torrents"), numberSelected]];
         
             uint64_t size = 0;
-            int fileCount = 0;
+            NSInteger fileCount = 0;
             NSEnumerator * enumerator = [torrents objectEnumerator];
             Torrent * torrent;
             while ((torrent = [enumerator nextObject]))
@@ -279,6 +283,7 @@ typedef enum
             [fUploadedTotalField setStringValue: @""];
             [fFailedHashField setStringValue: @""];
             [fDateActivityField setStringValue: @""];
+            [fRatioField setStringValue: @""];
             
             //options fields
             [fUploadLimitPopUp setEnabled: NO];
@@ -335,7 +340,6 @@ typedef enum
         
         [fStateField setStringValue: @""];
         [fProgressField setStringValue: @""];
-        [fRatioField setStringValue: @""];
         
         [fSwarmSpeedField setStringValue: @""];
         [fErrorMessageView setString: @""];
@@ -409,7 +413,7 @@ typedef enum
         if ([torrent isFolder])
         {
             NSString * fileString;
-            int fileCount = [torrent fileCount];
+            NSInteger fileCount = [torrent fileCount];
             if (fileCount == 1)
                 fileString = NSLocalizedString(@"1 file", "Inspector -> selected torrents");
             else
@@ -440,12 +444,12 @@ typedef enum
         {
             NSString * location = [torrent publicTorrentLocation];
             [fTorrentLocationField setStringValue: [location stringByAbbreviatingWithTildeInPath]];
-            [fTorrentLocationField setToolTip: [location stringByAppendingFormat: @"\n\n%@", [torrent torrentLocation]]];
+            [fTorrentLocationField setToolTip: location];
         }
         else
         {
-            [fTorrentLocationField setStringValue: NSLocalizedString(@"Transmission Support Folder", "Inspector -> torrent location")];
-            [fTorrentLocationField setToolTip: [torrent torrentLocation]];
+            [fTorrentLocationField setStringValue: @""];
+            [fTorrentLocationField setToolTip: nil];
         }
         
         [fDateAddedField setObjectValue: [torrent dateAdded]];
@@ -529,10 +533,10 @@ typedef enum
     NSEnumerator * enumerator = [fTorrents objectEnumerator];
     Torrent * torrent = [enumerator nextObject]; //first torrent
     
-    int uploadSpeedMode = [torrent speedMode: YES],
-        uploadSpeedLimit = [torrent speedLimit: YES],
-        downloadSpeedMode = [torrent speedMode: NO],
-        downloadSpeedLimit = [torrent speedLimit: NO];
+    NSInteger uploadSpeedMode = [torrent speedMode: YES],
+                uploadSpeedLimit = [torrent speedLimit: YES],
+                downloadSpeedMode = [torrent speedMode: NO],
+                downloadSpeedLimit = [torrent speedLimit: NO];
     
     while ((torrent = [enumerator nextObject])
             && (uploadSpeedMode != INVALID || uploadSpeedLimit != INVALID
@@ -552,7 +556,7 @@ typedef enum
     }
     
     //set upload view
-    int index;
+    NSInteger index;
     if (uploadSpeedMode == TR_SPEEDLIMIT_SINGLE)
         index = OPTION_POPUP_LIMIT;
     else if (uploadSpeedMode == TR_SPEEDLIMIT_UNLIMITED)
@@ -594,8 +598,8 @@ typedef enum
     enumerator = [fTorrents objectEnumerator];
     torrent = [enumerator nextObject]; //first torrent
     
-    int checkRatio = [torrent ratioSetting];
-    float ratioLimit = [torrent ratioLimit];
+    NSInteger checkRatio = [torrent ratioSetting];
+    CGFloat ratioLimit = [torrent ratioLimit];
     
     while ((torrent = [enumerator nextObject]) && (checkRatio != INVALID || checkRatio != INVALID))
     {
@@ -628,7 +632,7 @@ typedef enum
     enumerator = [fTorrents objectEnumerator];
     torrent = [enumerator nextObject]; //first torrent
     
-    int maxPeers = [torrent maxPeerConnect];
+    NSInteger maxPeers = [torrent maxPeerConnect];
     
     while ((torrent = [enumerator nextObject]))
     {
@@ -679,7 +683,7 @@ typedef enum
 
 - (void) setTab: (id) sender
 {
-    int oldTabTag = fCurrentTabTag;
+    NSInteger oldTabTag = fCurrentTabTag;
     fCurrentTabTag = [fTabMatrix selectedTag];
     if (fCurrentTabTag == oldTabTag)
         return;
@@ -687,7 +691,7 @@ typedef enum
     [self updateInfoStats];
     
     //take care of old view
-    float oldHeight = 0;
+    CGFloat oldHeight = 0;
     NSString * oldResizeSaveKey = nil;
     if (oldTabTag != INVALID)
     {
@@ -785,12 +789,12 @@ typedef enum
     
     if (resizeSaveKey)
     {
-        float height = [[NSUserDefaults standardUserDefaults] floatForKey: resizeSaveKey];
-        if (height != 0)
+        CGFloat height = [[NSUserDefaults standardUserDefaults] floatForKey: resizeSaveKey];
+        if (height != 0.0)
             viewRect.size.height = MAX(height, TAB_MIN_HEIGHT);
     }
     
-    float difference = (viewRect.size.height - oldHeight) * [window userSpaceScaleFactor];
+    CGFloat difference = (viewRect.size.height - oldHeight) * [window userSpaceScaleFactor];
     windowRect.origin.y -= difference;
     windowRect.size.height += difference;
     
@@ -821,7 +825,7 @@ typedef enum
 
 - (void) setNextTab
 {
-    int tag = [fTabMatrix selectedTag]+1;
+    NSInteger tag = [fTabMatrix selectedTag]+1;
     if (tag >= [fTabMatrix numberOfColumns])
         tag = 0;
     
@@ -831,7 +835,7 @@ typedef enum
 
 - (void) setPreviousTab
 {
-    int tag = [fTabMatrix selectedTag]-1;
+    NSInteger tag = [fTabMatrix selectedTag]-1;
     if (tag < 0)
         tag = [fTabMatrix numberOfColumns]-1;
     
@@ -839,7 +843,7 @@ typedef enum
     [self setTab: nil];
 }
 
-- (int) numberOfRowsInTableView: (NSTableView *) tableView
+- (NSInteger) numberOfRowsInTableView: (NSTableView *) tableView
 {
     if (tableView == fPeerTable)
         return fPeers ? [fPeers count] : 0;
@@ -850,7 +854,7 @@ typedef enum
     return 0;
 }
 
-- (id) tableView: (NSTableView *) tableView objectValueForTableColumn: (NSTableColumn *) column row: (int) row
+- (id) tableView: (NSTableView *) tableView objectValueForTableColumn: (NSTableColumn *) column row: (NSInteger) row
 {
     if (tableView == fPeerTable)
     {
@@ -894,7 +898,7 @@ typedef enum
         id item = [fTrackers objectAtIndex: row];
         if ([item isKindOfClass: [NSNumber class]])
         {
-            int tier = [item intValue];
+            NSInteger tier = [item intValue];
             if (tier == 0)
                 return NSLocalizedString(@"User-Added", "Inspector -> tracker table");
             else
@@ -931,7 +935,7 @@ typedef enum
     else;
 }
 
-- (BOOL) tableView: (NSTableView *) tableView shouldSelectRow: (int) row
+- (BOOL) tableView: (NSTableView *) tableView shouldSelectRow: (NSInteger) row
 {
     return tableView == fTrackerTable;
 }
@@ -940,7 +944,7 @@ typedef enum
 {
     if ([notification object] == fTrackerTable)
     {
-        int numSelected = [fTrackerTable numberOfSelectedRows];
+        NSInteger numSelected = [fTrackerTable numberOfSelectedRows];
         [fTrackerAddRemoveControl setEnabled: numSelected > 0 forSegment: TRACKER_REMOVE_TAG];
     }
 }
@@ -953,7 +957,7 @@ typedef enum
 }
 
 - (NSString *) tableView: (NSTableView *) tableView toolTipForCell: (NSCell *) cell rect: (NSRectPointer) rect
-                tableColumn: (NSTableColumn *) column row: (int) row mouseLocation: (NSPoint) mouseLocation
+                tableColumn: (NSTableColumn *) column row: (NSInteger) row mouseLocation: (NSPoint) mouseLocation
 {
     if (tableView == fPeerTable)
     {
@@ -967,7 +971,7 @@ typedef enum
             [components addObject: NSLocalizedString(@"Encrypted Connection", "Inspector -> Peers tab -> table row tooltip")];
         
         NSString * portString;
-        int port;
+        NSInteger port;
         if ((port = [[peer objectForKey: @"Port"] intValue]) > 0)
             portString = [NSString stringWithFormat: @"%d", port];
         else
@@ -1087,9 +1091,8 @@ typedef enum
     NSString * folder = [torrent downloadFolder];
     NSIndexSet * indexes = [fileOutlineView selectedRowIndexes];
     NSMutableArray * urlArray = [NSMutableArray arrayWithCapacity: [indexes count]];
-
-    int i;
-    for (i = [indexes firstIndex]; i != NSNotFound; i = [indexes indexGreaterThanIndex: i])
+    
+    for (NSUInteger i = [indexes firstIndex]; i != NSNotFound; i = [indexes indexGreaterThanIndex: i])
     {
         FileListNode * item = [fileOutlineView itemAtRow: i];
         if ([self canQuickLookFile: item])
@@ -1104,8 +1107,7 @@ typedef enum
     FileOutlineView * fileOutlineView = [fFileController outlineView];
     NSIndexSet * indexes = [fileOutlineView selectedRowIndexes];
     
-    int i;
-    for (i = [indexes firstIndex]; i != NSNotFound; i = [indexes indexGreaterThanIndex: i])
+    for (NSUInteger i = [indexes firstIndex]; i != NSNotFound; i = [indexes indexGreaterThanIndex: i])
         if ([self canQuickLookFile: [fileOutlineView itemAtRow: i]])
             return YES;
     
@@ -1120,8 +1122,7 @@ typedef enum
     NSString * folder = [[fTorrents objectAtIndex: 0] downloadFolder];
     NSRange visibleRows = [fileOutlineView rowsInRect: [fileOutlineView bounds]];
     
-    int row;
-    for (row = visibleRows.location; row < NSMaxRange(visibleRows); row++)
+    for (NSUInteger row = visibleRows.location; row < NSMaxRange(visibleRows); row++)
     {
         FileListNode * rowItem = [fileOutlineView itemAtRow: row];
         if ([[folder stringByAppendingPathComponent: [rowItem fullPath]] isEqualToString: fullPath])
@@ -1166,7 +1167,7 @@ typedef enum
 - (void) setSpeedMode: (id) sender
 {
     BOOL upload = sender == fUploadLimitPopUp;
-    int mode;
+    NSInteger mode;
     switch ([sender indexOfSelectedItem])
     {
         case OPTION_POPUP_LIMIT:
@@ -1204,7 +1205,7 @@ typedef enum
 - (void) setSpeedLimit: (id) sender
 {
     BOOL upload = sender == fUploadLimitField;
-    int limit = [sender intValue];
+    NSInteger limit = [sender intValue];
     
     Torrent * torrent;
     NSEnumerator * enumerator = [fTorrents objectEnumerator];
@@ -1215,7 +1216,7 @@ typedef enum
 
 - (void) setRatioSetting: (id) sender
 {
-    int setting;
+    NSInteger setting;
     switch ([sender indexOfSelectedItem])
     {
         case OPTION_POPUP_LIMIT:
@@ -1249,7 +1250,7 @@ typedef enum
 
 - (void) setRatioLimit: (id) sender
 {
-    float limit = [sender floatValue];
+    CGFloat limit = [sender floatValue];
     
     Torrent * torrent;
     NSEnumerator * enumerator = [fTorrents objectEnumerator];
@@ -1259,7 +1260,7 @@ typedef enum
 
 - (void) setPeersConnectLimit: (id) sender
 {
-    int limit = [sender intValue];
+    NSInteger limit = [sender intValue];
     
     Torrent * torrent;
     NSEnumerator * enumerator = [fTorrents objectEnumerator];
@@ -1308,7 +1309,7 @@ typedef enum
 
 - (void) updateInfoActivity
 {
-    int numberSelected = [fTorrents count];
+    NSInteger numberSelected = [fTorrents count];
     if (numberSelected == 0)
         return;
     
@@ -1373,6 +1374,11 @@ typedef enum
         
         [fPiecesView updateView];
     }
+    else if (numberSelected > 1)
+    {
+        [fRatioField setStringValue: [NSString stringForRatio: tr_getRatio(uploadedTotal, downloadedTotal)]];
+    }
+    else;
 }
 
 #warning reload table when necessary?
@@ -1394,7 +1400,7 @@ typedef enum
     [fAnnounceResponseField setToolTip: announceResponse];
     [fAnnounceResponseField setSelectable: ![announceResponse isEqualToString: @""]];
     
-    int announceNext = [torrent nextAnnounceTime];
+    NSInteger announceNext = [torrent nextAnnounceTime];
     NSString * announceNextString;
     switch (announceNext)
     {
@@ -1429,7 +1435,7 @@ typedef enum
     [fScrapeResponseField setToolTip: scrapeResponse];
     [fScrapeResponseField setSelectable: ![scrapeResponse isEqualToString: @""]];
     
-    int scrapeNext = [torrent nextScrapeTime];
+    NSInteger scrapeNext = [torrent nextScrapeTime];
     NSString * scrapeNextString;
     switch (scrapeNext)
     {
@@ -1451,7 +1457,7 @@ typedef enum
         return;
     Torrent * torrent = [fTorrents objectAtIndex: 0];
     
-    int seeders = [torrent seeders], leechers = [torrent leechers], completed = [torrent completedFromTracker];
+    NSInteger seeders = [torrent seeders], leechers = [torrent leechers], completed = [torrent completedFromTracker];
     [fSeedersField setStringValue: seeders >= 0 ? [NSString stringWithFormat: @"%d", seeders] : @""];
     [fLeechersField setStringValue: leechers >= 0 ? [NSString stringWithFormat: @"%d", leechers] : @""];
     [fCompletedFromTrackerField setStringValue: completed >= 0 ? [NSString stringWithFormat: @"%d", completed] : @""];
@@ -1460,14 +1466,14 @@ typedef enum
     
     if (active)
     {
-        int total = [torrent totalPeersConnected];
+        NSInteger total = [torrent totalPeersConnected];
         NSString * connected = [NSString stringWithFormat:
                                 NSLocalizedString(@"%d Connected", "Inspector -> Peers tab -> peers"), total];
         
         if (total > 0)
         {
             NSMutableArray * components = [NSMutableArray arrayWithCapacity: 4];
-            int count;
+            NSInteger count;
             if ((count = [torrent totalPeersTracker]) > 0)
                 [components addObject: [NSString stringWithFormat:
                                         NSLocalizedString(@"%d tracker", "Inspector -> Peers tab -> peers"), count]];
@@ -1516,7 +1522,7 @@ typedef enum
         [fFileController reloadData];
 }
 
-- (NSView *) tabViewForTag: (int) tag
+- (NSView *) tabViewForTag: (NSInteger) tag
 {
     switch (tag)
     {
@@ -1554,7 +1560,7 @@ typedef enum
     
     if (hide)
     {
-        float webSeedFrameMaxY = NSMaxY(webSeedFrame);
+        CGFloat webSeedFrameMaxY = NSMaxY(webSeedFrame);
         webSeedFrame.size.height = 0;
         webSeedFrame.origin.y = webSeedFrameMaxY;
         
@@ -1631,30 +1637,27 @@ typedef enum
     if (![[NSFileManager defaultManager] fileExistsAtPath: [[torrent downloadFolder] stringByAppendingPathComponent: [item fullPath]]])
         return NO;
     
-    return [item isFolder] || [torrent fileProgress: [[item indexes] firstIndex]] == 1.0;
+    return [item isFolder] || [torrent fileProgress: item] == 1.0;
 }
 
 - (void) addTrackers
 {
     [[self window] makeKeyWindow];
     
-    int i;
+    NSUInteger index = 1;
     if ([[fTorrents objectAtIndex: 0] hasAddedTrackers])
     {
-        for (i = 1; i < [fTrackers count]; i++)
-            if ([[fTrackers objectAtIndex: i] isKindOfClass: [NSNumber class]])
+        for (; index < [fTrackers count]; index++)
+            if ([[fTrackers objectAtIndex: index] isKindOfClass: [NSNumber class]])
                 break;
     }
     else
-    {
         [fTrackers insertObject: [NSNumber numberWithInt: 0] atIndex: 0];
-        i = 1;
-    }
     
-    [fTrackers insertObject: @"" atIndex: i];
+    [fTrackers insertObject: @"" atIndex: index];
     [fTrackerTable reloadData];
-    [fTrackerTable selectRow: i byExtendingSelection: NO];
-    [fTrackerTable editColumn: 0 row: i withEvent: nil select: YES];
+    [fTrackerTable selectRow: index byExtendingSelection: NO];
+    [fTrackerTable editColumn: 0 row: index withEvent: nil select: YES];
 }
 
 - (void) removeTrackers
