@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: PiecesView.m 6215 2008-06-17 17:17:15Z livings124 $
+ * $Id: PiecesView.m 7018 2008-11-02 14:04:57Z livings124 $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -23,11 +23,13 @@
  *****************************************************************************/
 
 #import "PiecesView.h"
+#import "Torrent.h"
 #import "InfoWindowController.h"
 #import "CTGradient.h"
+#import "utils.h"
 
 #define MAX_ACROSS 18
-#define BETWEEN 1.0
+#define BETWEEN 1.0f
 
 #define HIGH_PEERS 30
 
@@ -45,14 +47,14 @@
     fBack = [[NSImage alloc] initWithSize: [self bounds].size];
     
     [fBack lockFocus];
-    CTGradient * gradient = [CTGradient gradientWithBeginningColor: [NSColor colorWithCalibratedWhite: 0.0 alpha: 0.4]
-                                endingColor: [NSColor colorWithCalibratedWhite: 0.2 alpha: 0.4]];
-    [gradient fillRect: [self bounds] angle: 90.0];
+    CTGradient * gradient = [CTGradient gradientWithBeginningColor: [NSColor colorWithCalibratedWhite: 0.0f alpha: 0.4f]
+                                endingColor: [NSColor colorWithCalibratedWhite: 0.2f alpha: 0.4f]];
+    [gradient fillRect: [self bounds] angle: 90.0f];
     [fBack unlockFocus];
     
     //store box colors
-    fGreenAvailabilityColor = [[NSColor colorWithCalibratedRed: 0.0 green: 1.0 blue: 0.4 alpha: 1.0] retain];
-    fBluePieceColor = [[NSColor colorWithCalibratedRed: 0.0 green: 0.4 blue: 0.8 alpha: 1.0] retain];
+    fGreenAvailabilityColor = [[NSColor colorWithCalibratedRed: 0.0f green: 1.0f blue: 0.4f alpha: 1.0f] retain];
+    fBluePieceColor = [[NSColor colorWithCalibratedRed: 0.0f green: 0.4f blue: 0.8f alpha: 1.0f] retain];
             
     //actually draw the box
     [self setTorrent: nil];
@@ -81,7 +83,7 @@
         fNumPieces = MIN([fTorrent pieceCount], MAX_ACROSS * MAX_ACROSS);
         fAcross = ceil(sqrt(fNumPieces));
         
-        float width = [self bounds].size.width;
+        CGFloat width = [self bounds].size.width;
         fWidth = (width - (fAcross + 1) * BETWEEN) / fAcross;
         fExtraBorder = (width - ((fWidth + BETWEEN) * fAcross + BETWEEN)) / 2;
     }
@@ -130,12 +132,12 @@
     
     NSImage * image = [self image];
     
-    int i, j, index = -1;
+    NSInteger index = -1;
     NSRect rect = NSMakeRect(0, 0, fWidth, fWidth);
     BOOL change = NO;
     
-    for (i = 0; i < fAcross; i++)
-        for (j = 0; j < fAcross; j++)
+    for (NSInteger i = 0; i < fAcross; i++)
+        for (NSInteger j = 0; j < fAcross; j++)
         {
             index++;
             if (index >= fNumPieces)
@@ -146,7 +148,7 @@
             
             NSColor * pieceColor = nil;
             
-            if (showAvailablity ? pieces[index] == -1 : piecesPercent[index] == 1.0)
+            if (showAvailablity ? pieces[index] == -1 : piecesPercent[index] == 1.0f)
             {
                 if (first || fPieces[index] != PIECE_FINISHED)
                 {
@@ -162,7 +164,7 @@
                     }
                 }
             }
-            else if (showAvailablity ? pieces[index] == 0 : piecesPercent[index] == 0.0)
+            else if (showAvailablity ? pieces[index] == 0 : piecesPercent[index] == 0.0f)
             {
                 if (first || fPieces[index] != PIECE_NONE)
                 {
@@ -181,7 +183,7 @@
             else
             {
                 //always redraw "mixed"
-                float percent = showAvailablity ? (float)pieces[index]/HIGH_PEERS : piecesPercent[index];
+                CGFloat percent = showAvailablity ? (CGFloat)pieces[index]/HIGH_PEERS : piecesPercent[index];
                 NSColor * fullColor = showAvailablity ? fGreenAvailabilityColor : fBluePieceColor;
                 pieceColor = [[NSColor whiteColor] blendedColorWithFraction: percent ofColor: fullColor];
                 fPieces[index] = PIECE_SOME;
