@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: PrefsController.h 6361 2008-07-19 13:40:23Z livings124 $
+ * $Id: PrefsController.h 7666 2009-01-11 18:04:45Z livings124 $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -24,13 +24,11 @@
 
 #import <Cocoa/Cocoa.h>
 #import <transmission.h>
-#import <Sparkle/Sparkle.h>
-#import "PortChecker.h"
+
+@class PortChecker;
 
 @interface PrefsController : NSWindowController
 {
-    tr_handle * fHandle;
-    
     NSUserDefaults * fDefaults;
     BOOL fHasLoaded;
     
@@ -40,13 +38,11 @@
     
     IBOutlet NSPopUpButton * fFolderPopUp, * fIncompleteFolderPopUp, * fImportFolderPopUp;
     IBOutlet NSTextField * fRatioStopField, * fQueueDownloadField, * fQueueSeedField, * fStalledField;
-    
-    SUUpdater * fUpdater;
 
     IBOutlet NSTextField * fUploadField, * fDownloadField,
                         * fSpeedLimitUploadField, * fSpeedLimitDownloadField;
     
-    IBOutlet NSTextField * fPeersGlobalField, * fPeersTorrentField, * fBlocklistMessageField;
+    IBOutlet NSTextField * fPeersGlobalField, * fPeersTorrentField, * fBlocklistMessageField, * fBlocklistDateField;
     IBOutlet NSButton * fBlocklistEnableCheck;
     
     PortChecker * fPortChecker;
@@ -61,16 +57,14 @@
     IBOutlet NSPopUpButton * fProxyTypePopUp;
     
     IBOutlet NSTextField * fRPCPortField, * fRPCPasswordField;
-    IBOutlet NSTableView * fRPCAccessTable;
+    IBOutlet NSTableView * fRPCWhitelistTable;
+    NSMutableArray * fRPCWhitelistArray;
     IBOutlet NSSegmentedControl * fRPCAddRemoveControl;
-    NSMutableArray * fRPCAccessArray;
 }
 
-- (id) initWithHandle: (tr_handle *) handle;
-- (tr_handle *) handle;
-- (void) setUpdater: (SUUpdater *) updater;
++ (void) setHandle: (tr_session *) handle;
++ (tr_session *) handle;
 
-- (void) updatePortField;
 - (void) setPort: (id) sender;
 - (void) setNat: (id) sender;
 - (void) updatePortStatus;
@@ -88,11 +82,11 @@
 
 - (void) setBlocklistEnabled: (id) sender;
 - (void) updateBlocklist: (id) sender;
+- (void) setBlocklistAutoUpdate: (id) sender;
 - (void) updateBlocklistFields;
 
 - (void) setBadge: (id) sender;
 - (void) resetWarnings: (id) sender;
-- (void) setCheckForUpdate: (id) sender;
 
 - (void) setQueue: (id) sender;
 - (void) setQueueNumber: (id) sender;
@@ -138,7 +132,8 @@
 - (void) setRPCPassword: (id) sender;
 - (void) updateRPCPassword;
 - (void) setRPCPort: (id) sender;
-- (void) updateRPCAccessList;
+- (void) setRPCUseWhitelist: (id) sender;
+- (void) updateRPCWhitelist;
 - (void) addRemoveRPCIP: (id) sender;
 
 - (void) helpForPeers: (id) sender;
