@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: PrefsController.m 5742 2008-05-04 19:45:51Z livings124 $
+ * $Id: PrefsController.m 7672 2009-01-11 21:17:05Z livings124 $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -23,21 +23,38 @@
  *****************************************************************************/
 
 #import "PrefsController.h"
-#import "BlocklistDownloader.h"
+#import "BlocklistDownloaderViewController.h"
+#import "BlocklistScheduler.h"
+#import "PortChecker.h"
 #import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #import "UKKQueue.h"
+#import "utils.h"
 
 #define DOWNLOAD_FOLDER     0
 #define DOWNLOAD_TORRENT    2
 
-#define UPDATE_SECONDS 86400
+#define PROXY_HTTP      0
+#define PROXY_SOCKS4    1
+#define PROXY_SOCKS5    2
+
+#define RPC_IP_ADD_TAG      0
+#define RPC_IP_REMOVE_TAG   1
 
 #define TOOLBAR_GENERAL     @"TOOLBAR_GENERAL"
 #define TOOLBAR_TRANSFERS   @"TOOLBAR_TRANSFERS"
 #define TOOLBAR_BANDWIDTH   @"TOOLBAR_BANDWIDTH"
 #define TOOLBAR_PEERS       @"TOOLBAR_PEERS"
 #define TOOLBAR_NETWORK     @"TOOLBAR_NETWORK"
+#define TOOLBAR_REMOTE      @"TOOLBAR_REMOTE"
+
+#define PROXY_KEYCHAIN_SERVICE  "Transmission:Proxy"
+#define PROXY_KEYCHAIN_NAME     "Proxy"
+
+#define RPC_KEYCHAIN_SERVICE    "Transmission:Remote"
+#define RPC_KEYCHAIN_NAME       "Remote"
+
+#define WEBUI_URL   @"http://localhost:%d/transmission/web/"
 
 @interface PrefsController (Private)
 
@@ -47,26 +64,38 @@
 - (void) incompleteFolderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info;
 - (void) importFolderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info;
 
+- (void) setKeychainPassword: (const char *) password forService: (const char *) service username: (const char *) username;
+
 @end
 
 @implementation PrefsController
 
-- (id) initWithHandle: (tr_handle *) handle
+tr_session * fHandle;
++ (void) setHandle: (tr_session *) handle
+{
+    fHandle = handle;
+}
+
++ (tr_session *) handle
+{
+    return fHandle;
+}
+
+- (id) init
 {
     if ((self = [super initWithWindowNibName: @"PrefsWindow"]))
     {
         fDefaults = [NSUserDefaults standardUserDefaults];
-        fHandle = handle;
         
         //checks for old version speeds of -1
         if ([fDefaults integerForKey: @"UploadLimit"] < 0)
         {
-            [fDefaults setInteger: 20 forKey: @"UploadLimit"];
+            [fDefaults removeObjectForKey: @"UploadLimit"];
             [fDefaults setBool: NO forKey: @"CheckUpload"];
         }
         if ([fDefaults integerForKey: @"DownloadLimit"] < 0)
         {
-            [fDefaults setInteger: 20 forKey: @"DownloadLimit"];
+            [fDefaults removeObjectForKey: @"DownloadLimit"];
             [fDefaults setBool: NO forKey: @"CheckDownload"];
         }
         
@@ -80,39 +109,49 @@
             [fDefaults removeObjectForKey: @"DownloadChoice"];
         }
         
-        //set check for update to right value
-        [self setCheckForUpdate: nil];
-        
         //set auto import
         NSString * autoPath;
         if ([fDefaults boolForKey: @"AutoImport"] && (autoPath = [fDefaults stringForKey: @"AutoImportDirectory"]))
             [[UKKQueue sharedFileWatcher] addPath: [autoPath stringByExpandingTildeInPath]];
         
+        //set blocklist scheduler
+        [[BlocklistScheduler scheduler] updateSchedule];
+        
         //set encryption
         [self setEncryptionMode: nil];
         
         //actually set bandwidth limits
         [self applySpeedSettings: nil];
+        
+        //set proxy type
+        [self updateProxyType];
+        [self updateProxyPassword];
+        
+        //update rpc whitelist
+        [self updateRPCPassword];
+        
+        fRPCWhitelistArray = [[fDefaults arrayForKey: @"RPCWhitelist"] mutableCopy];
+        if (!fRPCWhitelistArray)
+            fRPCWhitelistArray = [[NSMutableArray arrayWithObject: @"127.0.0.1"] retain];
+        [self updateRPCWhitelist];
     }
     
     return self;
 }
 
-- (tr_handle *) handle
-{
-    return fHandle;
-}
-
 - (void) dealloc
 {
-    if (fPortStatusTimer)
-        [fPortStatusTimer invalidate];
+    [[NSNotificationCenter defaultCenter] removeObserver: self];
+    
+    [fPortStatusTimer invalidate];
     if (fPortChecker)
     {
         [fPortChecker cancelProbe];
         [fPortChecker release];
     }
     
+    [fRPCWhitelistArray release];
+    
     [super dealloc];
 }
 
@@ -131,6 +170,13 @@
     
     [self setPrefView: nil];
     
+    if (![NSApp isOnLeopardOrBetter])
+    {
+        [fRPCAddRemoveControl sizeToFit];
+        [fRPCAddRemoveControl setLabel: @"+" forSegment: RPC_IP_ADD_TAG];
+        [fRPCAddRemoveControl setLabel: @"-" forSegment: RPC_IP_REMOVE_TAG];
+    }
+    
     //set download folder
     [fFolderPopUp selectItemAtIndex: [fDefaults boolForKey: @"DownloadLocationConstant"] ? DOWNLOAD_FOLDER : DOWNLOAD_TORRENT];
     
@@ -161,13 +207,40 @@
     [fQueueSeedField setIntValue: [fDefaults integerForKey: @"QueueSeedNumber"]];
     [fStalledField setIntValue: [fDefaults integerForKey: @"StalledMinutes"]];
     
+    //set proxy type
+    [fProxyAddressField setStringValue: [fDefaults stringForKey: @"ProxyAddress"]];
+    int proxyType;
+    switch(tr_sessionGetProxyType(fHandle))
+    {
+        case TR_PROXY_SOCKS4:
+            proxyType = PROXY_SOCKS4;
+            break;
+        case TR_PROXY_SOCKS5:
+            proxyType = PROXY_SOCKS5;
+            break;
+        case TR_PROXY_HTTP:
+            proxyType = PROXY_HTTP;
+    }
+    [fProxyTypePopUp selectItemAtIndex: proxyType];
+    
+    //set proxy password - does NOT need to be released
+    [fProxyPasswordField setStringValue: [NSString stringWithUTF8String: tr_sessionGetProxyPassword(fHandle)]];
+    
+    //set proxy port
+    [fProxyPortField setIntValue: [fDefaults integerForKey: @"ProxyPort"]];
+    
     //set blocklist
     [self updateBlocklistFields];
-}
-
-- (void) setUpdater: (SUUpdater *) updater
-{
-    fUpdater = updater;
+    [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(updateBlocklistFields)
+        name: @"BlocklistUpdated" object: nil];
+    
+    //set rpc port
+    [fRPCPortField setIntValue: [fDefaults integerForKey: @"RPCPort"]];
+    
+    //set rpc password - has to be released
+    char * rpcPassword = tr_sessionGetRPCPassword(fHandle);
+    [fRPCPasswordField setStringValue: [NSString stringWithUTF8String: rpcPassword]];
+    tr_free(rpcPassword);
 }
 
 - (NSToolbarItem *) toolbar: (NSToolbar *) toolbar itemForItemIdentifier: (NSString *) ident willBeInsertedIntoToolbar: (BOOL) flag
@@ -214,6 +287,14 @@
         [item setAction: @selector(setPrefView:)];
         [item setAutovalidates: NO];
     }
+    else if ([ident isEqualToString: TOOLBAR_REMOTE])
+    {
+        [item setLabel: NSLocalizedString(@"Remote", "Preferences -> toolbar item title")];
+        [item setImage: [NSImage imageNamed: @"Remote.png"]];
+        [item setTarget: self];
+        [item setAction: @selector(setPrefView:)];
+        [item setAutovalidates: NO];
+    }
     else
     {
         [item release];
@@ -235,28 +316,23 @@
 
 - (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar *) toolbar
 {
-    return [NSArray arrayWithObjects: TOOLBAR_GENERAL, TOOLBAR_TRANSFERS, TOOLBAR_BANDWIDTH, TOOLBAR_PEERS, TOOLBAR_NETWORK, nil];
-}
-
-//used by ipc
-- (void) updatePortField
-{
-    [fPortField setIntValue: [fDefaults integerForKey: @"BindPort"]];
+    return [NSArray arrayWithObjects: TOOLBAR_GENERAL, TOOLBAR_TRANSFERS, TOOLBAR_BANDWIDTH,
+                                        TOOLBAR_PEERS, TOOLBAR_NETWORK, TOOLBAR_REMOTE, nil];
 }
 
 - (void) setPort: (id) sender
 {
     int port = [sender intValue];
     [fDefaults setInteger: port forKey: @"BindPort"];
-    tr_setBindPort(fHandle, port);
+    tr_sessionSetPeerPort(fHandle, port);
     
-    fPublicPort = -1;
+    fPeerPort = -1;
     [self updatePortStatus];
 }
 
 - (void) setNat: (id) sender
 {
-    tr_natTraversalEnable(fHandle, [fDefaults boolForKey: @"NatTraversal"]);
+    tr_sessionSetPortForwardingEnabled(fHandle, [fDefaults boolForKey: @"NatTraversal"]);
     
     fNatStatus = -1;
     [self updatePortStatus];
@@ -264,14 +340,17 @@
 
 - (void) updatePortStatus
 {
-    const tr_handle_status * stat = tr_handleStatus(fHandle);
-    if (fNatStatus != stat->natTraversalStatus || fPublicPort != stat->publicPort)
+    const tr_port_forwarding fwd = tr_sessionGetPortForwarding(fHandle);
+    const int port = tr_sessionGetPeerPort(fHandle);
+    BOOL natStatusChanged = (fNatStatus != fwd);
+    BOOL peerPortChanged = (fPeerPort != port);
+
+    if (natStatusChanged || peerPortChanged)
     {
-        fNatStatus = stat->natTraversalStatus;
-        fPublicPort = stat->publicPort;
+        fNatStatus = fwd;
+        fPeerPort = port;
         
-        [fPortStatusField setStringValue: [NSLocalizedString(@"Checking port status", "Preferences -> Network -> port status")
-            stringByAppendingEllipsis]];
+        [fPortStatusField setStringValue: @""];
         [fPortStatusImage setImage: nil];
         [fPortStatusProgress startAnimation: self];
         
@@ -280,7 +359,8 @@
             [fPortChecker cancelProbe];
             [fPortChecker release];
         }
-        fPortChecker = [[PortChecker alloc] initForPort: fPublicPort withDelegate: self];
+        BOOL delay = natStatusChanged || tr_sessionIsPortForwardingEnabled(fHandle);
+        fPortChecker = [[PortChecker alloc] initForPort: fPeerPort delay: delay withDelegate: self];
     }
 }
 
@@ -297,13 +377,8 @@
             [fPortStatusField setStringValue: NSLocalizedString(@"Port is closed", "Preferences -> Network -> port status")];
             [fPortStatusImage setImage: [NSImage imageNamed: @"RedDot.png"]];
             break;
-        case PORT_STATUS_STEALTH:
-            [fPortStatusField setStringValue: NSLocalizedString(@"Port is stealth", "Preferences -> Network -> port status")];
-            [fPortStatusImage setImage: [NSImage imageNamed: @"RedDot.png"]];
-            break;
         case PORT_STATUS_ERROR:
-            [fPortStatusField setStringValue: NSLocalizedString(@"Unable to check port status",
-                                                "Preferences -> Network -> port status")];
+            [fPortStatusField setStringValue: NSLocalizedString(@"Port check site is down", "Preferences -> Network -> port status")];
             [fPortStatusImage setImage: [NSImage imageNamed: @"YellowDot.png"]];
             break;
     }
@@ -350,7 +425,7 @@
 {
     int count = [sender intValue];
     [fDefaults setInteger: count forKey: @"PeersTotal"];
-    tr_setGlobalPeerLimit(fHandle, count);
+    tr_sessionSetPeerLimit(fHandle, count);
 }
 
 - (void) setPeersTorrent: (id) sender
@@ -361,13 +436,13 @@
 
 - (void) setPEX: (id) sender
 {
-    tr_setPexEnabled(fHandle, [fDefaults boolForKey: @"PEXGlobal"]);
+    tr_sessionSetPexEnabled(fHandle, [fDefaults boolForKey: @"PEXGlobal"]);
 }
 
 - (void) setEncryptionMode: (id) sender
 {
-    tr_setEncryptionMode(fHandle, [fDefaults boolForKey: @"EncryptionPrefer"] ? 
-        ([fDefaults boolForKey: @"EncryptionRequire"] ? TR_ENCRYPTION_REQUIRED : TR_ENCRYPTION_PREFERRED) : TR_PLAINTEXT_PREFERRED);
+    tr_sessionSetEncryption(fHandle, [fDefaults boolForKey: @"EncryptionPrefer"] ? 
+        ([fDefaults boolForKey: @"EncryptionRequire"] ? TR_ENCRYPTION_REQUIRED : TR_ENCRYPTION_PREFERRED) : TR_CLEAR_PREFERRED);
 }
 
 - (void) setBlocklistEnabled: (id) sender
@@ -375,11 +450,18 @@
     BOOL enable = [sender state] == NSOnState;
     [fDefaults setBool: enable forKey: @"Blocklist"];
     tr_blocklistSetEnabled(fHandle, enable);
+    
+    [[BlocklistScheduler scheduler] updateSchedule];
 }
 
 - (void) updateBlocklist: (id) sender
 {
-    [BlocklistDownloader downloadWithPrefsController: self];
+    [BlocklistDownloaderViewController downloadWithPrefsController: self];
+}
+
+- (void) setBlocklistAutoUpdate: (id) sender
+{
+    [[BlocklistScheduler scheduler] updateSchedule];
 }
 
 - (void) updateBlocklistFields
@@ -403,25 +485,47 @@
     
     [fBlocklistEnableCheck setEnabled: exists];
     [fBlocklistEnableCheck setState: exists && [fDefaults boolForKey: @"Blocklist"]];
+    
+    NSString * updatedDateString;
+    if (exists)
+    {
+        NSDate * updatedDate = [fDefaults objectForKey: @"BlocklistLastUpdate"];
+        if (updatedDate)
+        {
+            NSDateFormatter * dateFormatter = [[NSDateFormatter alloc] init];
+            [dateFormatter setDateStyle: NSDateFormatterFullStyle];
+            [dateFormatter setTimeStyle: NSDateFormatterShortStyle];
+            
+            updatedDateString = [dateFormatter stringFromDate: updatedDate];
+            [dateFormatter release];
+        }
+        else
+            updatedDateString = NSLocalizedString(@"N/A", "Prefs -> blocklist -> message");
+    }
+    else
+        updatedDateString = NSLocalizedString(@"Never", "Prefs -> blocklist -> message");
+    
+    [fBlocklistDateField setStringValue: [NSString stringWithFormat: @"%@: %@",
+        NSLocalizedString(@"Last updated", "Prefs -> blocklist -> message"), updatedDateString]];
 }
 
 - (void) applySpeedSettings: (id) sender
 {
     if ([fDefaults boolForKey: @"SpeedLimit"])
     {
-        tr_setUseGlobalSpeedLimit(fHandle, TR_UP, 1);
-        tr_setGlobalSpeedLimit(fHandle, TR_UP, [fDefaults integerForKey: @"SpeedLimitUploadLimit"]);
+        tr_sessionSetSpeedLimitEnabled(fHandle, TR_UP, 1);
+        tr_sessionSetSpeedLimit(fHandle, TR_UP, [fDefaults integerForKey: @"SpeedLimitUploadLimit"]);
         
-        tr_setUseGlobalSpeedLimit(fHandle, TR_DOWN, 1);
-        tr_setGlobalSpeedLimit(fHandle, TR_DOWN, [fDefaults integerForKey: @"SpeedLimitDownloadLimit"]);
+        tr_sessionSetSpeedLimitEnabled(fHandle, TR_DOWN, 1);
+        tr_sessionSetSpeedLimit(fHandle, TR_DOWN, [fDefaults integerForKey: @"SpeedLimitDownloadLimit"]);
     }
     else
     {
-        tr_setUseGlobalSpeedLimit(fHandle, TR_UP, [fDefaults boolForKey: @"CheckUpload"]);
-        tr_setGlobalSpeedLimit(fHandle, TR_UP, [fDefaults integerForKey: @"UploadLimit"]);
+        tr_sessionSetSpeedLimitEnabled(fHandle, TR_UP, [fDefaults boolForKey: @"CheckUpload"]);
+        tr_sessionSetSpeedLimit(fHandle, TR_UP, [fDefaults integerForKey: @"UploadLimit"]);
         
-        tr_setUseGlobalSpeedLimit(fHandle, TR_DOWN, [fDefaults boolForKey: @"CheckDownload"]);
-        tr_setGlobalSpeedLimit(fHandle, TR_DOWN, [fDefaults integerForKey: @"DownloadLimit"]);
+        tr_sessionSetSpeedLimitEnabled(fHandle, TR_DOWN, [fDefaults boolForKey: @"CheckDownload"]);
+        tr_sessionSetSpeedLimit(fHandle, TR_DOWN, [fDefaults integerForKey: @"DownloadLimit"]);
     }
 }
 
@@ -500,18 +604,13 @@
 
 - (void) resetWarnings: (id) sender
 {
-    [fDefaults setBool: YES forKey: @"WarningDuplicate"];
-    [fDefaults setBool: YES forKey: @"WarningRemainingSpace"];
-    [fDefaults setBool: YES forKey: @"WarningFolderDataSameName"];
-    [fDefaults setBool: YES forKey: @"WarningResetStats"];
-}
-
-- (void) setCheckForUpdate: (id) sender
-{
-    NSTimeInterval seconds = [fDefaults boolForKey: @"CheckForUpdates"] ? UPDATE_SECONDS : 0;
-    [fDefaults setInteger: seconds forKey: @"SUScheduledCheckInterval"];
-    if (fUpdater)
-        [fUpdater scheduleCheckWithInterval: seconds];
+    [fDefaults removeObjectForKey: @"WarningDuplicate"];
+    [fDefaults removeObjectForKey: @"WarningRemainingSpace"];
+    [fDefaults removeObjectForKey: @"WarningFolderDataSameName"];
+    [fDefaults removeObjectForKey: @"WarningResetStats"];
+    [fDefaults removeObjectForKey: @"WarningCreatorBlankAddress"];
+    [fDefaults removeObjectForKey: @"WarningRemoveBuiltInTracker"];
+    [fDefaults removeObjectForKey: @"WarningInvalidOpen"];
 }
 
 - (void) setQueue: (id) sender
@@ -608,6 +707,271 @@
     [[NSNotificationCenter defaultCenter] postNotificationName: @"AutoSizeSettingChange" object: self];
 }
 
+- (void) setProxyEnabled: (id) sender
+{
+    tr_sessionSetProxyEnabled(fHandle, [fDefaults boolForKey: @"Proxy"]);
+}
+
+- (void) setProxyAddress: (id) sender
+{
+    NSString * address = [sender stringValue];
+    tr_sessionSetProxy(fHandle, [address UTF8String]);
+    [fDefaults setObject: address forKey: @"ProxyAddress"];
+}
+
+- (void) setProxyPort: (id) sender
+{
+    int port = [sender intValue];
+    [fDefaults setInteger: port forKey: @"ProxyPort"];
+    tr_sessionSetProxyPort(fHandle, port);
+}
+
+- (void) setProxyType: (id) sender
+{
+    NSString * type;
+    switch ([sender indexOfSelectedItem])
+    {
+        case PROXY_HTTP:
+            type = @"HTTP";
+            break;
+        case PROXY_SOCKS4:
+            type = @"SOCKS4";
+            break;
+        case PROXY_SOCKS5:
+            type = @"SOCKS5";
+    }
+    
+    [fDefaults setObject: type forKey: @"ProxyType"];
+    [self updateProxyType];
+}
+
+- (void) updateProxyType
+{
+    NSString * typeString = [fDefaults stringForKey: @"ProxyType"];
+    tr_proxy_type type;
+    if ([typeString isEqualToString: @"SOCKS4"])
+        type = TR_PROXY_SOCKS4;
+    else if ([typeString isEqualToString: @"SOCKS5"])
+        type = TR_PROXY_SOCKS5;
+    else
+    {
+        //safety
+        if (![typeString isEqualToString: @"HTTP"])
+        {
+            typeString = @"HTTP";
+            [fDefaults setObject: typeString forKey: @"ProxyType"];
+        }
+        type = TR_PROXY_HTTP;
+    }
+    
+    tr_sessionSetProxyType(fHandle, type);
+}
+
+- (void) setProxyAuthorize: (id) sender
+{
+    BOOL enable = [fDefaults boolForKey: @"ProxyAuthorize"];
+    tr_sessionSetProxyAuthEnabled(fHandle, enable);
+}
+
+- (void) setProxyUsername: (id) sender
+{
+    tr_sessionSetProxyUsername(fHandle, [[fDefaults stringForKey: @"ProxyUsername"] UTF8String]);
+}
+
+- (void) setProxyPassword: (id) sender
+{
+    const char * password = [[sender stringValue] UTF8String];
+    [self setKeychainPassword: password forService: PROXY_KEYCHAIN_SERVICE username: PROXY_KEYCHAIN_NAME];
+    
+    tr_sessionSetProxyPassword(fHandle, password);
+}
+
+- (void) updateProxyPassword
+{
+    UInt32 passwordLength;
+    const char * password = nil;
+    SecKeychainFindGenericPassword(NULL, strlen(PROXY_KEYCHAIN_SERVICE), PROXY_KEYCHAIN_SERVICE,
+        strlen(PROXY_KEYCHAIN_NAME), PROXY_KEYCHAIN_NAME, &passwordLength, (void **)&password, NULL);
+    
+    if (password != NULL)
+    {
+        char fullPassword[passwordLength+1];
+        strncpy(fullPassword, password, passwordLength);
+        fullPassword[passwordLength] = '\0';
+        SecKeychainItemFreeContent(NULL, (void *)password);
+        
+        tr_sessionSetProxyPassword(fHandle, fullPassword);
+        [fProxyPasswordField setStringValue: [NSString stringWithUTF8String: fullPassword]];
+    }
+}
+
+- (void) setRPCEnabled: (id) sender
+{
+    tr_sessionSetRPCEnabled(fHandle, [fDefaults boolForKey: @"RPC"]);
+}
+
+- (void) linkWebUI: (id) sender
+{
+    NSString * urlString = [NSString stringWithFormat: WEBUI_URL, [fDefaults integerForKey: @"RPCPort"]];
+    [[NSWorkspace sharedWorkspace] openURL: [NSURL URLWithString: urlString]];
+}
+
+- (void) setRPCAuthorize: (id) sender
+{
+    tr_sessionSetRPCPasswordEnabled(fHandle, [fDefaults boolForKey: @"RPCAuthorize"]);
+}
+
+- (void) setRPCUsername: (id) sender
+{
+    tr_sessionSetRPCUsername(fHandle, [[fDefaults stringForKey: @"RPCUsername"] UTF8String]);
+}
+
+- (void) setRPCPassword: (id) sender
+{
+    const char * password = [[sender stringValue] UTF8String];
+    [self setKeychainPassword: password forService: RPC_KEYCHAIN_SERVICE username: RPC_KEYCHAIN_NAME];
+    
+    tr_sessionSetRPCPassword(fHandle, password);
+}
+
+- (void) updateRPCPassword
+{
+    UInt32 passwordLength;
+    const char * password = nil;
+    SecKeychainFindGenericPassword(NULL, strlen(RPC_KEYCHAIN_SERVICE), RPC_KEYCHAIN_SERVICE,
+        strlen(RPC_KEYCHAIN_NAME), RPC_KEYCHAIN_NAME, &passwordLength, (void **)&password, NULL);
+    
+    if (password != NULL)
+    {
+        char fullPassword[passwordLength+1];
+        strncpy(fullPassword, password, passwordLength);
+        fullPassword[passwordLength] = '\0';
+        SecKeychainItemFreeContent(NULL, (void *)password);
+        
+        tr_sessionSetRPCPassword(fHandle, fullPassword);
+        [fRPCPasswordField setStringValue: [NSString stringWithUTF8String: fullPassword]];
+    }
+}
+
+- (void) setRPCPort: (id) sender
+{
+    int port = [sender intValue];
+    [fDefaults setInteger: port forKey: @"RPCPort"];
+    tr_sessionSetRPCPort(fHandle, port);
+}
+
+- (void) setRPCUseWhitelist: (id) sender
+{
+    tr_sessionSetRPCWhitelistEnabled(fHandle, [fDefaults boolForKey: @"RPCUseWhitelist"]);
+}
+
+- (void) updateRPCWhitelist
+{
+    NSString * string = [fRPCWhitelistArray componentsJoinedByString: @","];
+    tr_sessionSetRPCWhitelist(fHandle, [string UTF8String]);
+}
+
+- (void) addRemoveRPCIP: (id) sender
+{
+    //don't allow add/remove when currently adding - it leads to weird results
+    if ([fRPCWhitelistTable editedRow] != -1)
+        return;
+    
+    if ([[sender cell] tagForSegment: [sender selectedSegment]] == RPC_IP_REMOVE_TAG)
+    {
+        [fRPCWhitelistArray removeObjectsAtIndexes: [fRPCWhitelistTable selectedRowIndexes]];
+        [fRPCWhitelistTable deselectAll: self];
+        [fRPCWhitelistTable reloadData];
+        
+        [fDefaults setObject: fRPCWhitelistArray forKey: @"RPCWhitelist"];
+        [self updateRPCWhitelist];
+    }
+    else
+    {
+        [fRPCWhitelistArray addObject: @""];
+        [fRPCWhitelistTable reloadData];
+        
+        int row = [fRPCWhitelistArray count] - 1;
+        [fRPCWhitelistTable selectRow: row byExtendingSelection: NO];
+        [fRPCWhitelistTable editColumn: 0 row: row withEvent: nil select: YES];
+    }
+}
+
+- (NSInteger) numberOfRowsInTableView: (NSTableView *) tableView
+{
+    return [fRPCWhitelistArray count];
+}
+
+- (id) tableView: (NSTableView *) tableView objectValueForTableColumn: (NSTableColumn *) tableColumn row: (NSInteger) row
+{
+    return [fRPCWhitelistArray objectAtIndex: row];
+}
+
+- (void) tableView: (NSTableView *) tableView setObjectValue: (id) object forTableColumn: (NSTableColumn *) tableColumn
+    row: (NSInteger) row
+{
+    NSArray * components = [object componentsSeparatedByString: @"."];
+    NSMutableArray * newComponents = [NSMutableArray arrayWithCapacity: 4];
+        
+    //create better-formatted ip string
+    BOOL valid = false;
+    if ([components count] == 4)
+    {
+        valid = true;
+        NSEnumerator * enumerator = [components objectEnumerator];
+        NSString * component;
+        while ((component = [enumerator nextObject]))
+        {
+            if ([component isEqualToString: @"*"])
+                [newComponents addObject: component];
+            else
+            {
+                int num = [component intValue];
+                if (num >= 0 && num < 256)
+                    [newComponents addObject: [[NSNumber numberWithInt: num] stringValue]];
+                else
+                {
+                    valid = false;
+                    break;
+                }
+            }
+        }
+    }
+    
+    NSString * newIP;
+    if (valid)
+    {
+        newIP = [newComponents componentsJoinedByString: @"."];
+        
+        //don't allow the same ip address
+        if ([fRPCWhitelistArray containsObject: newIP] && ![[fRPCWhitelistArray objectAtIndex: row] isEqualToString: newIP])
+            valid = false;
+    }
+    
+    if (valid)
+    {
+        [fRPCWhitelistArray replaceObjectAtIndex: row withObject: newIP];
+        [fRPCWhitelistArray sortUsingSelector: @selector(compareNumeric:)];
+    }
+    else
+    {
+        NSBeep();
+        if ([[fRPCWhitelistArray objectAtIndex: row] isEqualToString: @""])
+            [fRPCWhitelistArray removeObjectAtIndex: row];
+    }
+        
+    [fRPCWhitelistTable deselectAll: self];
+    [fRPCWhitelistTable reloadData];
+    
+    [fDefaults setObject: fRPCWhitelistArray forKey: @"RPCWhitelist"];
+    [self updateRPCWhitelist];
+}
+
+- (void) tableViewSelectionDidChange: (NSNotification *) notification
+{
+    [fRPCAddRemoveControl setEnabled: [fRPCWhitelistTable numberOfSelectedRows] > 0 forSegment: RPC_IP_REMOVE_TAG];
+}
+
 - (void) helpForPeers: (id) sender
 {
     [[NSHelpManager sharedHelpManager] openHelpAnchor: @"PeersPrefs"
@@ -620,26 +984,111 @@
         inBook: [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleHelpBookName"]];
 }
 
+- (void) helpForRemote: (id) sender
+{
+    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"RemotePrefs"
+        inBook: [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleHelpBookName"]];
+}
+
+- (void) rpcUpdatePrefs
+{
+    //encryption
+    tr_encryption_mode encryptionMode = tr_sessionGetEncryption(fHandle);
+    [fDefaults setBool: encryptionMode != TR_CLEAR_PREFERRED forKey: @"EncryptionPrefer"];
+    [fDefaults setBool: encryptionMode == TR_ENCRYPTION_REQUIRED forKey: @"EncryptionRequire"];
+    
+    //download directory
+    NSString * downloadLocation = [[NSString stringWithUTF8String: tr_sessionGetDownloadDir(fHandle)] stringByStandardizingPath];
+    [fDefaults setObject: downloadLocation forKey: @"DownloadFolder"];
+    
+    //peers
+    uint16_t peersTotal = tr_sessionGetPeerLimit(fHandle);
+    [fDefaults setInteger: peersTotal forKey: @"PeersTotal"];
+    
+    //pex
+    BOOL pex = tr_sessionIsPexEnabled(fHandle);
+    [fDefaults setBool: pex forKey: @"PEXGlobal"];
+    
+    //port
+    int port = tr_sessionGetPeerPort(fHandle);
+    [fDefaults setInteger: port forKey: @"BindPort"];
+    
+    BOOL nat = tr_sessionIsPortForwardingEnabled(fHandle);
+    [fDefaults setBool: nat forKey: @"NatTraversal"];
+    
+    fPeerPort = -1;
+    fNatStatus = -1;
+    [self updatePortStatus];
+    
+    //speed limit - down
+    BOOL downLimitEnabled = tr_sessionIsSpeedLimitEnabled(fHandle, TR_DOWN);
+    [fDefaults setBool: downLimitEnabled forKey: @"CheckDownload"];
+    
+    int downLimit = tr_sessionGetSpeedLimit(fHandle, TR_DOWN);
+    [fDefaults setInteger: downLimit forKey: @"DownloadLimit"];
+    
+    //speed limit - up
+    BOOL upLimitEnabled = tr_sessionIsSpeedLimitEnabled(fHandle, TR_UP);
+    [fDefaults setBool: upLimitEnabled forKey: @"CheckUpload"];
+    
+    int upLimit = tr_sessionGetSpeedLimit(fHandle, TR_UP);
+    [fDefaults setInteger: upLimit forKey: @"UploadLimit"];
+    
+    //update gui if loaded
+    if (fHasLoaded)
+    {
+        //encryption handled by bindings
+        
+        //download directory handled by bindings
+        
+        [fPeersGlobalField setIntValue: peersTotal];
+        
+        //pex handled by bindings
+        
+        [fPortField setIntValue: port];
+        //port forwarding (nat) handled by bindings
+        
+        //limit check handled by bindings
+        [fDownloadField setIntValue: downLimit];
+        
+        //limit check handled by bindings
+        [fUploadField setIntValue: upLimit];
+    }
+}
+
 @end
 
 @implementation PrefsController (Private)
 
 - (void) setPrefView: (id) sender
 {
-    NSView * view = fGeneralView;
+    NSString * identifier;
     if (sender)
     {
-        NSString * identifier = [sender itemIdentifier];
-        if ([identifier isEqualToString: TOOLBAR_TRANSFERS])
-            view = fTransfersView;
-        else if ([identifier isEqualToString: TOOLBAR_BANDWIDTH])
-            view = fBandwidthView;
-        else if ([identifier isEqualToString: TOOLBAR_PEERS])
-            view = fPeersView;
-        else if ([identifier isEqualToString: TOOLBAR_NETWORK])
-            view = fNetworkView;
-        else; //general view already selected
+        identifier = [sender itemIdentifier];
+        [[NSUserDefaults standardUserDefaults] setObject: identifier forKey: @"SelectedPrefView"];
     }
+    else
+        identifier = [[NSUserDefaults standardUserDefaults] stringForKey: @"SelectedPrefView"];
+    
+    NSView * view;
+    if ([identifier isEqualToString: TOOLBAR_TRANSFERS])
+        view = fTransfersView;
+    else if ([identifier isEqualToString: TOOLBAR_BANDWIDTH])
+        view = fBandwidthView;
+    else if ([identifier isEqualToString: TOOLBAR_PEERS])
+        view = fPeersView;
+    else if ([identifier isEqualToString: TOOLBAR_NETWORK])
+        view = fNetworkView;
+    else if ([identifier isEqualToString: TOOLBAR_REMOTE])
+        view = fRemoteView;
+    else
+    {
+        identifier = TOOLBAR_GENERAL; //general view is the default selected
+        view = fGeneralView;
+    }
+    
+    [[[self window] toolbar] setSelectedItemIdentifier: identifier];
     
     NSWindow * window = [self window];
     if ([window contentView] == view)
@@ -720,4 +1169,39 @@
     [fImportFolderPopUp selectItemAtIndex: 0];
 }
 
+- (void) setKeychainPassword: (const char *) password forService: (const char *) service username: (const char *) username
+{
+    SecKeychainItemRef item = NULL;
+    NSUInteger passwordLength = strlen(password);
+    
+    OSStatus result = SecKeychainFindGenericPassword(NULL, strlen(service), service, strlen(username), username, NULL, NULL, &item);
+    if (result == noErr && item)
+    {
+        if (passwordLength > 0) //found, so update
+        {
+            result = SecKeychainItemModifyAttributesAndData(item, NULL, passwordLength, (const void *)password);
+            if (result != noErr)
+                NSLog(@"Problem updating Keychain item: %s", GetMacOSStatusErrorString(result));
+        }
+        else //remove the item
+        {
+            result = SecKeychainItemDelete(item);
+            if (result != noErr)
+                NSLog(@"Problem removing Keychain item: %s", GetMacOSStatusErrorString(result));
+        }
+    }
+    else if (result == errSecItemNotFound) //not found, so add
+    {
+        if (passwordLength > 0)
+        {
+            result = SecKeychainAddGenericPassword(NULL, strlen(service), service, strlen(username), username,
+                        passwordLength, (const void *)password, NULL);
+            if (result != noErr)
+                NSLog(@"Problem adding Keychain item: %s", GetMacOSStatusErrorString(result));
+        }
+    }
+    else
+        NSLog(@"Problem accessing Keychain: %s", GetMacOSStatusErrorString(result));
+}
+
 @end
