@@ -43,11 +43,11 @@
 #ifndef WIN32
 #include <sys/socket.h>
 #include <sys/wait.h>
-#include <sys/signal.h>
-#include <assert.h>
+#include <signal.h>
 #include <unistd.h>
 #include <netdb.h>
 #endif
+#include <assert.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <stdlib.h>
@@ -61,7 +61,9 @@
 #include "log.h"
 
 #include "regress.h"
+#ifndef WIN32
 #include "regress.gen.h"
+#endif
 
 int pair[2];
 int test_ok;
@@ -179,7 +181,7 @@ timeout_cb(int fd, short event, void *arg)
 	struct timeval tv;
 	int diff;
 
-	gettimeofday(&tcalled, NULL);
+	evutil_gettimeofday(&tcalled, NULL);
 	if (evutil_timercmp(&tcalled, &tset, >))
 		evutil_timersub(&tcalled, &tset, &tv);
 	else
@@ -442,7 +444,7 @@ test_simpletimeout(void)
 	evtimer_set(&ev, timeout_cb, NULL);
 	evtimer_add(&ev, &tv);
 
-	gettimeofday(&tset, NULL);
+	evutil_gettimeofday(&tset, NULL);
 	event_dispatch();
 
 	cleanup_test();
@@ -450,11 +452,25 @@ test_simpletimeout(void)
 
 #ifndef WIN32
 extern struct event_base *current_base;
+
+static void
+child_signal_cb(int fd, short event, void *arg)
+{
+	struct timeval tv;
+	int *pint = arg;
+
+	*pint = 1;
+
+	tv.tv_usec = 500000;
+	tv.tv_sec = 0;
+	event_loopexit(&tv);
+}
+
 static void
 test_fork(void)
 {
-	int status;
-	struct event ev;
+	int status, got_sigchld = 0;
+	struct event ev, sig_ev;
 	pid_t pid;
 
 	setup_test("After fork: ");
@@ -465,6 +481,9 @@ test_fork(void)
 	if (event_add(&ev, NULL) == -1)
 		exit(1);
 
+	signal_set(&sig_ev, SIGCHLD, child_signal_cb, &got_sigchld);
+	signal_add(&sig_ev, NULL);
+
 	if ((pid = fork()) == 0) {
 		/* in the child */
 		if (event_reinit(current_base) == -1) {
@@ -472,6 +491,8 @@ test_fork(void)
 			exit(1);
 		}
 
+		signal_del(&sig_ev);
+
 		called = 0;
 
 		event_dispatch();
@@ -503,6 +524,13 @@ test_fork(void)
 
 	event_dispatch();
 
+	if (!got_sigchld) {
+		fprintf(stdout, "FAILED (sigchld)\n");
+		exit(1);
+	}
+
+	signal_del(&sig_ev);
+
 	cleanup_test();
 }
 
@@ -515,6 +543,9 @@ test_simplesignal(void)
 	setup_test("Simple signal: ");
 	signal_set(&ev, SIGALRM, signal_cb, &ev);
 	signal_add(&ev, NULL);
+	/* find bugs in which operations are re-ordered */
+	signal_del(&ev);
+	signal_add(&ev, NULL);
 
 	memset(&itv, 0, sizeof(itv));
 	itv.it_value.tv_sec = 1;
@@ -529,6 +560,36 @@ test_simplesignal(void)
 	cleanup_test();
 }
 
+static void
+test_multiplesignal(void)
+{
+	struct event ev_one, ev_two;
+	struct itimerval itv;
+
+	setup_test("Multiple signal: ");
+
+	signal_set(&ev_one, SIGALRM, signal_cb, &ev_one);
+	signal_add(&ev_one, NULL);
+
+	signal_set(&ev_two, SIGALRM, signal_cb, &ev_two);
+	signal_add(&ev_two, NULL);
+
+	memset(&itv, 0, sizeof(itv));
+	itv.it_value.tv_sec = 1;
+	if (setitimer(ITIMER_REAL, &itv, NULL) == -1)
+		goto skip_simplesignal;
+
+	event_dispatch();
+
+ skip_simplesignal:
+	if (signal_del(&ev_one) == -1)
+		test_ok = 0;
+	if (signal_del(&ev_two) == -1)
+		test_ok = 0;
+
+	cleanup_test();
+}
+
 static void
 test_immediatesignal(void)
 {
@@ -703,6 +764,52 @@ out:
 	cleanup_test();
 	return;
 }
+
+void
+signal_cb_swp(int sig, short event, void *arg)
+{
+	called++;
+	if (called < 5)
+		raise(sig);
+	else
+		event_loopexit(NULL);
+}
+void
+timeout_cb_swp(int fd, short event, void *arg)
+{
+	if (called == -1) {
+		struct timeval tv = {5, 0};
+
+		called = 0;
+		evtimer_add((struct event *)arg, &tv);
+		raise(SIGUSR1);
+		return;
+	}
+	test_ok = 0;
+	event_loopexit(NULL);
+}
+
+static void
+test_signal_while_processing(void)
+{
+	struct event_base *base = event_init();
+	struct event ev, ev_timer;
+	struct timeval tv = {0, 0};
+
+	setup_test("Receiving a signal while processing other signal: ");
+
+	called = -1;
+	test_ok = 1;
+	signal_set(&ev, SIGUSR1, signal_cb_swp, NULL);
+	signal_add(&ev, NULL);
+	evtimer_set(&ev_timer, timeout_cb_swp, &ev_timer);
+	evtimer_add(&ev_timer, &tv);
+	event_dispatch();
+
+	event_base_free(base);
+	cleanup_test();
+	return;
+}
 #endif
 
 static void
@@ -760,9 +867,9 @@ test_loopexit(void)
 	tv.tv_sec = 1;
 	event_loopexit(&tv);
 
-	gettimeofday(&tv_start, NULL);
+	evutil_gettimeofday(&tv_start, NULL);
 	event_dispatch();
-	gettimeofday(&tv_end, NULL);
+	evutil_gettimeofday(&tv_end, NULL);
 	evutil_timersub(&tv_end, &tv_start, &tv_end);
 
 	evtimer_del(&ev);
@@ -1062,7 +1169,7 @@ test_priorities(int npriorities)
 	struct test_pri_event one, two;
 	struct timeval tv;
 
-	snprintf(buf, sizeof(buf), "Testing Priorities %d: ", npriorities);
+	evutil_snprintf(buf, sizeof(buf), "Testing Priorities %d: ", npriorities);
 	setup_test(buf);
 
 	event_base_priority_init(global_base, npriorities);
@@ -1327,6 +1434,7 @@ evtag_test(void)
 	fprintf(stdout, "OK\n");
 }
 
+#ifndef WIN32
 static void
 rpc_test(void)
 {
@@ -1352,7 +1460,7 @@ rpc_test(void)
 	EVTAG_ASSIGN(attack, weapon, "feather");
 	EVTAG_ASSIGN(attack, action, "tickle");
 
-	gettimeofday(&tv_start, NULL);
+	evutil_gettimeofday(&tv_start, NULL);
 	for (i = 0; i < 1000; ++i) {
 		run = EVTAG_ADD(msg, run);
 		if (run == NULL) {
@@ -1360,6 +1468,8 @@ rpc_test(void)
 			exit(1);
 		}
 		EVTAG_ASSIGN(run, how, "very fast but with some data in it");
+		EVTAG_ASSIGN(run, fixed_bytes,
+		    (unsigned char*)"012345678901234567890123");
 	}
 
 	if (msg_complete(msg) == -1) {
@@ -1385,7 +1495,7 @@ rpc_test(void)
 		exit(1);
 	}
 
-	gettimeofday(&tv_end, NULL);
+	evutil_gettimeofday(&tv_end, NULL);
 	evutil_timersub(&tv_end, &tv_start, &tv_end);
 	fprintf(stderr, "(%.1f us/add) ",
 	    (float)tv_end.tv_sec/(float)i * 1000000.0 +
@@ -1410,6 +1520,7 @@ rpc_test(void)
 
 	fprintf(stdout, "OK\n");
 }
+#endif
 
 static void
 test_evutil_strtoll(void)
@@ -1450,6 +1561,10 @@ main (int argc, char **argv)
 	err = WSAStartup( wVersionRequested, &wsaData );
 #endif
 
+#ifndef WIN32
+	if (signal(SIGPIPE, SIG_IGN) == SIG_ERR)
+		return (1);
+#endif
 	setvbuf(stdout, NULL, _IONBF, 0);
 
 	/* Initalize the event library */
@@ -1474,7 +1589,9 @@ main (int argc, char **argv)
 
 	http_suite();
 
+#ifndef WIN32
 	rpc_suite();
+#endif
 
 	dns_suite();
 	
@@ -1495,6 +1612,7 @@ main (int argc, char **argv)
 	test_simpletimeout();
 #ifndef WIN32
 	test_simplesignal();
+	test_multiplesignal();
 	test_immediatesignal();
 #endif
 	test_loopexit();
@@ -1508,14 +1626,15 @@ main (int argc, char **argv)
 
 	evtag_test();
 
+#ifndef WIN32
 	rpc_test();
 
-#ifndef WIN32
 	test_signal_dealloc();
 	test_signal_pipeloss();
 	test_signal_switchbase();
 	test_signal_restore();
 	test_signal_assert();
+	test_signal_while_processing();
 #endif
 	
 	return (0);
