@@ -1,6 +1,6 @@
 /*
  * Xmission - a cross-platform bittorrent client
- * Copyright (C) 2007 Charles Kerr <charles@rebelbase.com>
+ * Copyright (C) 2007 Charles Kerr <charles@transmissionbt.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: speed-stats.cc 5975 2008-05-30 15:19:07Z charles $
+ * $Id: speed-stats.cc 7664 2009-01-11 17:46:51Z charles $
  */
 
 #include <wx/dcclient.h>
@@ -210,19 +210,19 @@ SpeedStats :: SetTorrent( tr_torrent * tor )
 }
 
 void
-SpeedStats :: Pulse( tr_handle * handle )
+SpeedStats :: Pulse( tr_session * handle )
 {
     // add a new record
-    float allUp, allDown;
-    tr_sessionGetSpeed( handle, &allDown, &allUp );
+    const double allUp   = tr_sessionGetPieceSpeed( handle, TR_UP );
+    const double allDown = tr_sessionGetPieceSpeed( handle, TR_DOWN );
     Speed s;
     s.time = time( NULL );
     s.allUp = allUp;
     s.allDown = allDown;
     if( myTorrent ) {
         const tr_stat * stat = tr_torrentStat( myTorrent );
-        s.torrentUp = stat->rateUpload;
-        s.torrentDown = stat->rateDownload;
+        s.torrentUp = stat->pieceUploadSpeed;
+        s.torrentDown = stat->pieceDownloadSpeed;
     }
     myStats.push_back( s );
 
