@@ -1,6 +1,6 @@
 /*
  * Xmission - a cross-platform bittorrent client
- * Copyright (C) 2007 Charles Kerr <charles@rebelbase.com>
+ * Copyright (C) 2007 Charles Kerr <charles@transmissionbt.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,11 +15,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: torrent-list.cc 6335 2008-07-15 18:24:18Z charles $
+ * $Id: torrent-list.cc 7664 2009-01-11 17:46:51Z charles $
  */
 
+#include <algorithm>
 #include <wx/intl.h>
 #include <torrent-list.h>
+#include <libtransmission/utils.h>
 
 namespace
 {
@@ -182,7 +184,7 @@ END_EVENT_TABLE()
 
 
 
-TorrentListCtrl :: TorrentListCtrl( tr_handle       * handle,
+TorrentListCtrl :: TorrentListCtrl( tr_session       * handle,
                                     wxConfig        * config,
                                     wxWindow        * parent,
                                     const wxPoint   & pos,
@@ -246,8 +248,8 @@ TorrentListCtrl :: RefreshTorrent( tr_torrent   * tor,
                 break;
 
             case COL_DOWNLOAD_SPEED:
-                if( s->rateDownload > 0.01 )
-                    xstr = getReadableSpeed( s->rateDownload );
+                if( s->pieceDownloadSpeed > 0.01 )
+                    xstr = getReadableSpeed( s->pieceDownloadSpeed );
                 else
                     xstr.Clear( );
                 break;
@@ -302,7 +304,7 @@ TorrentListCtrl :: RefreshTorrent( tr_torrent   * tor,
 
             case COL_STATE: /* FIXME: divine the meaning of these two columns */
             case COL_STATUS:
-                switch( s->status ) {
+                switch( s->activity ) {
                     case TR_STATUS_STOPPED:     xstr = _("Stopped"); break;
                     case TR_STATUS_CHECK:       xstr = wxString::Format ( _("Checking Files (%.0f)"), s->recheckProgress );  break;
                     case TR_STATUS_CHECK_WAIT:  xstr = _("Waiting to Check"); break;
@@ -317,8 +319,8 @@ TorrentListCtrl :: RefreshTorrent( tr_torrent   * tor,
                 break;
 
             case COL_UPLOAD_SPEED:
-                if( s->rateUpload > 0.01 )
-                    xstr = getReadableSpeed( s->rateUpload );
+                if( s->pieceUploadSpeed > 0.01 )
+                    xstr = getReadableSpeed( s->pieceUploadSpeed );
                 else
                     xstr.Clear( );
                 break;
@@ -418,9 +420,9 @@ TorrentListCtrl :: Compare( long item1, long item2, long sortData )
             break;
 
         case COL_DOWNLOAD_SPEED:
-            if( sa->rateDownload < sb->rateDownload )
+            if( sa->pieceDownloadSpeed < sb->pieceDownloadSpeed )
                 ret = -1;
-            else if( sa->rateDownload > sb->rateDownload )
+            else if( sa->pieceDownloadSpeed > sb->pieceDownloadSpeed )
                 ret =  1;
             else
                 ret = 0;
@@ -496,7 +498,7 @@ TorrentListCtrl :: Compare( long item1, long item2, long sortData )
 
         case COL_STATE: /* FIXME */
         case COL_STATUS:
-            ret = sa->status - sb->status;
+            ret = sa->activity - sb->activity;
             break;
 
         case COL_TOTAL:
@@ -504,9 +506,9 @@ TorrentListCtrl :: Compare( long item1, long item2, long sortData )
             break;
 
         case COL_UPLOAD_SPEED:
-            if( sa->rateUpload < sb->rateUpload )
+            if( sa->pieceUploadSpeed < sb->pieceUploadSpeed )
                 ret = -1;
-            else if( sa->rateUpload > sb->rateUpload )
+            else if( sa->pieceUploadSpeed > sb->pieceUploadSpeed )
                 ret = 1;
             else
                 ret = 0;
