@@ -1,6 +1,6 @@
 /*
  * Xmission - a cross-platform bittorrent client
- * Copyright (C) 2007 Charles Kerr <charles@rebelbase.com>
+ * Copyright (C) 2007 Charles Kerr <charles@transmissionbt.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: xmission.cc 4495 2008-01-05 18:17:23Z charles $
+ * $Id: xmission.cc 7664 2009-01-11 17:46:51Z charles $
  */
 
 #include <ctype.h>
@@ -40,6 +40,7 @@
 #include <wx/snglinst.h>
 #include <wx/splitter.h>
 #include <wx/statline.h>
+#include <wx/stdpaths.h>
 #include <wx/taskbar.h>
 #include <wx/tglbtn.h>
 #include <wx/toolbar.h>
@@ -52,6 +53,7 @@ extern "C"
 {
   #include <libtransmission/transmission.h>
   #include <libtransmission/utils.h>
+  #include <libtransmission/bencode.h>
 
   #include <images/play.xpm>
   #include <images/stop.xpm>
@@ -135,7 +137,7 @@ class MyApp : public wxApp
 
 namespace
 {
-    tr_handle * handle = NULL;
+    tr_session * handle = NULL;
 
     typedef std::vector<tr_torrent*> torrents_v;
 }
@@ -284,7 +286,7 @@ MyFrame :: OnStartUpdate( wxUpdateUIEvent& event )
 {
     bool enable = false;
     foreach( torrents_v, mySelectedTorrents, it )
-        if( tr_torrentStatCached(*it)->status == TR_STATUS_STOPPED )
+        if( tr_torrentStatCached(*it)->activity == TR_STATUS_STOPPED )
             enable = true;
     event.Enable( enable );
 }
@@ -292,7 +294,7 @@ void
 MyFrame :: OnStart( wxCommandEvent& WXUNUSED(unused) )
 {
     foreach( torrents_v, mySelectedTorrents, it )
-        if( tr_torrentStatCached(*it)->status == TR_STATUS_STOPPED )
+        if( tr_torrentStatCached(*it)->activity == TR_STATUS_STOPPED )
             tr_torrentStart( *it );
 }
 
@@ -304,7 +306,7 @@ MyFrame :: OnStopUpdate( wxUpdateUIEvent& event )
 {
     bool enable = false;
     foreach( torrents_v, mySelectedTorrents, it )
-        if( tr_torrentStatCached(*it)->status != TR_STATUS_STOPPED )
+        if( tr_torrentStatCached(*it)->activity != TR_STATUS_STOPPED )
             enable = true;
     event.Enable( enable );
 }
@@ -312,7 +314,7 @@ void
 MyFrame :: OnStop( wxCommandEvent& WXUNUSED(unused) )
 {
     foreach( torrents_v, mySelectedTorrents, it )
-        if( tr_torrentStat(*it)->status != TR_STATUS_STOPPED )
+        if( tr_torrentStat(*it)->activity != TR_STATUS_STOPPED )
             tr_torrentStop( *it );
 }
 
@@ -327,10 +329,8 @@ MyFrame :: OnRemoveUpdate( wxUpdateUIEvent& event )
 void
 MyFrame :: OnRemove( wxCommandEvent& WXUNUSED(unused) )
 {
-    foreach( torrents_v, mySelectedTorrents, it ) {
-        tr_torrentRemoveSaved( *it );
-        tr_torrentClose( *it );
-    }
+    foreach( torrents_v, mySelectedTorrents, it )
+        tr_torrentRemove( *it );
 }
 
 /**
@@ -345,7 +345,7 @@ void
 MyFrame :: OnRecheck( wxCommandEvent& WXUNUSED(unused) )
 {
     foreach( torrents_v, mySelectedTorrents, it )
-        tr_torrentRecheck( *it );
+        tr_torrentVerify( *it );
 }
 
 /**
@@ -372,7 +372,7 @@ void MyFrame :: OnOpen( wxCommandEvent& WXUNUSED(event) )
             const std::string filename = toStr( paths[i] );
             tr_ctor * ctor = tr_ctorNew( handle );
             tr_ctorSetMetainfoFromFile( ctor, filename.c_str() );
-            tr_ctorSetDestination( ctor, TR_FALLBACK, mySavePath.c_str() );
+            tr_ctorSetDownloadDir( ctor, TR_FALLBACK, mySavePath.c_str() );
             tr_torrent * tor = tr_torrentNew( handle, ctor, NULL );
             tr_ctorFree( ctor );
 
@@ -391,7 +391,16 @@ void MyFrame :: OnOpen( wxCommandEvent& WXUNUSED(event) )
 bool
 MyApp :: OnInit( )
 {
-    handle = tr_init( "wx" );
+    tr_benc settings;
+    const char * configDir;
+
+    tr_bencInitDict( &settings, 0 );
+    tr_sessionGetDefaultSettings( &settings );
+    configDir = tr_getDefaultConfigDir( "xmission" );
+
+    handle = tr_sessionInit( "wx", configDir, true, &settings );
+    
+    tr_bencFree( &settings );
 
     wxCmdLineParser cmdParser( cmdLineDesc, argc, argv );
     if( cmdParser.Parse ( ) )
@@ -457,7 +466,7 @@ void
 MyFrame :: OnPulse(wxTimerEvent& WXUNUSED(event) )
 {
     if( myExitTime ) {
-        if ( !tr_torrentCount(handle) ||  myExitTime<time(0) ) {
+        if ( !tr_sessionCountTorrents(handle) ||  myExitTime<time(0) ) {
             delete myTrayIcon;
             myTrayIcon = 0;
             Destroy( );
@@ -470,8 +479,8 @@ MyFrame :: OnPulse(wxTimerEvent& WXUNUSED(event) )
 
     mySpeedStats->Pulse( handle );
 
-    float down, up;
-    tr_torrentRates( handle, &down, &up );
+    const double up   = tr_sessionGetPieceSpeed( handle, TR_UP );
+    const double down = tr_sessionGetPieceSpeed( handle, TR_DOWN );
     wxString xstr = _("Total DL: ");
     xstr += getReadableSpeed( down );
     SetStatusText( xstr, 1 );
@@ -516,9 +525,9 @@ MyFrame :: MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size
 
     long port;
     wxString key = _T("port");
-    if( !myConfig->Read( key, &port, TR_DEFAULT_PORT ) )
+    if( !myConfig->Read( key, &port, atoi( TR_DEFAULT_PEER_PORT_STR ) ) )
         myConfig->Write( key, port );
-    tr_setBindPort( handle, port );
+    tr_sessionSetPeerPort( handle, port );
 
     key = _T("save-path");
     wxString wxstr;
@@ -681,8 +690,8 @@ MyFrame :: MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size
     int count = 0;
     tr_ctor * ctor = tr_ctorNew( handle );
     tr_ctorSetPaused( ctor, TR_FORCE, paused );
-    tr_ctorSetDestination( ctor, TR_FALLBACK, mySavePath.c_str() );
-    tr_torrent ** torrents = tr_loadTorrents ( handle, ctor, &count );
+    tr_ctorSetDownloadDir( ctor, TR_FALLBACK, mySavePath.c_str() );
+    tr_torrent ** torrents = tr_sessionLoadTorrents( handle, ctor, &count );
     myTorrents.insert( myTorrents.end(), torrents, torrents+count );
     tr_free( torrents );
     tr_ctorFree( ctor );
@@ -700,7 +709,7 @@ MyFrame :: OnExit( wxCommandEvent& WXUNUSED( event ) )
     mySelectedTorrents.clear( );
     ApplyCurrentFilter( );
 
-    tr_close( handle );
+    tr_sessionClose( handle );
 
     /* give the connections a max of 10 seconds to shut themselves down */
     myExitTime = time(0) + 10;
